/*==========================================================================
**
**	rlk/pdk_psdb.h
**
**	PSDB extensions for RAINlink in PDK
**
**==========================================================================
**
**	(c) Copyright 1996, 1997 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	ram	01/05/99	Created.
**
**==========================================================================
*/


#ifndef PDK_PSDB_H
#define PDK_PSDB_H

#include <pdk_types.h>
#include <pdk_constant.h>

#define UCHAR uchar
#define UINT32 uint32_t

#include <psdb.h>

typedef struct {
	PSDB_RHDR	ppr_header;
	union {
		uint32_t		ppr_dev_id_u;
		uint32_t		ppr_ppa_u;
	} ppr_dev_u;
} pdk_psdb_rhdr_t;

#define ppr_dev_id ppr_dev_u.ppr_dev_id_u
#define ppr_ppa ppr_dev_u.ppr_ppa_u

/****************************  RAIN IO layer *****************************/
#define PSDB_TYPE_IO_OPEN PSDB_RECORD_TYPE(PSDB_CLASS_PDK_IO, 1)
typedef struct {
	pdk_psdb_rhdr_t	pioo_header;
	int32_t		pioo_flags;
} pdk_psdb_io_open_t;

#define PSDB_TYPE_IO_CLOSE PSDB_RECORD_TYPE(PSDB_CLASS_PDK_IO, 2)
typedef struct {
	pdk_psdb_rhdr_t	pioc_header;
} pdk_psdb_io_close_t;

#define PSDB_TYPE_IO_GET PSDB_RECORD_TYPE(PSDB_CLASS_PDK_IO, 3)
typedef struct {
	pdk_psdb_rhdr_t piog_header;
	int32_t		piog_len;
} pdk_psdb_io_get_t;

/***************************  RAIN Link layer ***************************/

/* Variable length PPA lists use xxx_ppa_array[1] to reference the first
 * of N entries.
 */
#define PSDB_TYPE_CREATE_TRUNK PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 1)
typedef struct {
	pdk_psdb_rhdr_t	pct_header;
	int32_t		pct_num_ppa;
	int32_t		pct_ppa_array[1];
} pdk_psdb_create_trunk_t;

#define PSDB_TYPE_TEARDOWN_TRUNK PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 2)
typedef struct {
	pdk_psdb_rhdr_t	ptt_header;
} pdk_psdb_teardown_trunk_t;

#define PSDB_TYPE_CREATE_FAILOVER PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 3)
typedef struct {
	pdk_psdb_rhdr_t	pcf_header;
	int32_t		pcf_num_ppa;
	int32_t		pcf_ppa_array[1];
} pdk_psdb_create_failover_t;

#define PSDB_TYPE_TEARDOWN_FAILOVER PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 4)
typedef struct {
	pdk_psdb_rhdr_t	ptf_header;
} pdk_psdb_teardown_failover_t;

/* Configuration may be obtained on a per-device basis or a per-ppa basis, but
 * not both.  The unused field should be set to -1.  If both the pcr_dev_id and
 * pcr_ppa are -1, a list of all configurations for all PPA's is returned.
 *
 * If the MAX_PORTS is smaller than the number of ports actually configured in
 * the driver, rc_overflow will be set upon return.  The num_xxxx_ports will
 * reflect the actual number of entries in the arrays, not the number contained
 * in the driver(ie it should never be greater than MAX_PORTS).  If an
 * overflow is encountered, offset_ports can be used to get the next set
 * of ports.  When there are no more blocks of ports to be retreived overflow
 * will be set to 0.  rc_num_ports must be set to MAX_PORTS.
 */
#define PSDB_MAX_PORTS 64
typedef struct {
	int32_t			rc_version;
#define RC_VERSION 4
	int32_t			rc_ppa;
	int32_t			rc_linkup;
	uint32_t		rc_mode;
	uint32_t		rc_rx_timeout;
	int32_t			rc_num_primary_ports;
	int32_t			rc_num_configured_ports;
	int32_t			rc_num_active_ports;
	int32_t			rc_failover_ppa;	/* Failover primary */
	int32_t			rc_failover_port;	/* Active Failover */
	int32_t			rc_num_ports;		/* set to MAX_PORTS */
	int32_t			rc_offset_ports;
	int32_t			rc_overflow;
	int32_t			rc_primary_ports[PSDB_MAX_PORTS];
	int32_t			rc_configured_ports[PSDB_MAX_PORTS];
	int32_t			rc_active_ports[PSDB_MAX_PORTS];
} rlk_configuration_t;

/* Configuration may be obtained on a per-device basis or a per-ppa basis, but
 * not both.  The unused field should be set to -1.
 */
#define PSDB_TYPE_CONFIGURATION PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 5)
typedef struct {
	pdk_psdb_rhdr_t		pc_header;
	int32_t			pc_ppa;
	uint32_t		pc_type;
	rlk_configuration_t	pc_configuration;
} pdk_psdb_configuration_t;

/* Configuration device translation types */
#define PSDB_CONFIG_TRUNK_TYPE		1
#define PSDB_CONFIG_FAILOVER_TYPE	2

#define PSDB_TYPE_SET_MODE PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 6)
typedef struct {
	pdk_psdb_rhdr_t	psm_header;
	uint32_t	psm_mode;
	uint32_t	psm_time;
} pdk_psdb_set_mode_t;

/* PDK_RLK_MODEs */
#define RLK_LAYER_2_MODE 0x1
#define RLK_LAYER_3_MODE 0x2
#define RLK_TIMEOUT_MODE 0x4
#define RLK_ROUND_ROBIN_MODE 0x8
#define RLK_IP_TRUNKING_MODE 0x10
#define RLK_BALANCE_MODE 0x20
#define RLK_INTEGRITY_MODE 0x40
#define RLK_NOTIFICATION_MODE 0x80
#define RLK_HUB_MODE 0x100
#define RLK_SNAP_MODE 0x200

#define PSDB_TYPE_CLEAR_MODE PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 7)
typedef struct {
	pdk_psdb_rhdr_t	pcm_header;
	uint32_t	pcm_mode;
} pdk_psdb_clear_mode_t;

/* Cause connection to failover to another specified link.  If pf_to_ppa is
 * -1 it will go to another up link */
#define PSDB_TYPE_FAILOVER PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 8)
typedef struct {
	pdk_psdb_rhdr_t	pf_header;
	int32_t		pf_to_ppa;
} pdk_psdb_failover_t;

/* Notification of failover message.  Sent up from RLK layer.  See
 * RLK_NOTIFICATOIN_MODE.  If to_ppa is -1, the last link for
 * a failover group has gone down.  If from_ppa is -1, the failover group has
 * gone from no up links to being up on the to_ppa link
 */
#define PSDB_TYPE_FAILOVER_NOTIFICATION PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 10)
typedef struct {
	pdk_psdb_rhdr_t	pfn_header;
	int32_t		pfn_from_ppa;
	int32_t		pfn_to_ppa;
} pdk_psdb_failover_notification_t;

#define PSDB_TYPE_FAULT_NOTIFICATION PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 11)
typedef struct {
	pdk_psdb_rhdr_t	pftn_header;
	int32_t		pftn_fault_ppa;
	uint32_t	pftn_internal_fault;  /* 0 indicates external fault */
	uint32_t	pftn_on;
} pdk_psdb_fault_notification_t;

#define PSDB_TYPE_ERROR PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 100)
typedef struct {
	pdk_psdb_rhdr_t	pe_header;
	uint32_t	pe_error_type;
	uint32_t	pe_error;
} pdk_psdb_error_t;

#define PSDB_TYPE_SUCCESS PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 101)
typedef struct {
	pdk_psdb_rhdr_t	ps_header;
	uint32_t	ps_success_type;
	uint32_t	ps_pad;	/* error and success should be same size */
} pdk_psdb_success_t;

#define PSDB_TYPE_TEST PSDB_RECORD_TYPE(PSDB_CLASS_PDK_RLK, 200)
typedef struct {
	pdk_psdb_rhdr_t	pt_header;
	uint32_t	pt_test;
} pdk_psdb_test_t;

/***************************  HAL layer ***************************/

typedef struct hal_state_s {
	uint32_t	hs_version;
#define HS_VERSION 2
	uint32_t	hs_state;
	uint32_t	hs_fault;
	uint32_t	hs_link_cfg;	/* auto,10 half,... */
	uint32_t	hs_link_state;	/* down, 10.5 10.full 100.5 100 full */
	uint32_t	hs_tx_resources_max;	/* max number of transmit buffers */
	uint32_t	hs_tx_resources;	/* current number of transmit buffers */
	uint32_t	hs_tx_lwm;		/* low water mark number of transmit buffers - cleared on read */
	uint32_t	hs_tx_depleted;		/* number of times all transmit buffers consumed */
	uint32_t	hs_tx_packets;
	uint32_t	hs_tx_descriptor_count;
	uint32_t	hs_tx_err_packets;
	uint32_t	hs_tx_one_collision;
	uint32_t	hs_tx_more_collisions;
	uint32_t	hs_tx_max_collisions;
	uint32_t	hs_tx_late_collisions;
	uint32_t	hs_tx_carrier_loss;
	uint32_t	hs_tx_no_carrier;
	uint32_t	hs_tx_jabber;
	uint32_t	hs_tx_underrun;
	uint32_t	hs_tx_overrun;
	uint32_t	hs_tx_deferred;
	uint32_t	hs_rx_resources_max;	/* max number of receive buffers */
	uint32_t	hs_rx_resources;	/* current number of receive buffers */
	uint32_t	hs_rx_lwm;		/* low water mark number of receive buffers - cleared on read */
	uint32_t	hs_rx_depleted;		/* number of times all receive buffers consumed */
	uint32_t	hs_rx_packets;
	uint32_t	hs_rx_descriptor_count;
	uint32_t	hs_rx_err_packets;
	uint32_t	hs_rx_collisions;
	uint32_t	hs_rx_mii_error;
	uint32_t	hs_rx_crc;
	uint32_t	hs_rx_descriptor_error;
	uint32_t	hs_rx_frame_too_long;
	uint32_t	hs_rx_runt_frame;
	uint32_t	hs_rx_alignment_error;
	uint32_t	hs_rx_overrun;
	uint32_t	hs_tx_icount;
	uint32_t	hs_rx_icount;
	uint32_t	hs_other_icount;
} hal_state_t;

#define PSDB_TYPE_HAL_GET_STATE PSDB_RECORD_TYPE(PSDB_CLASS_PDK_HAL, 1)

typedef struct {
	pdk_psdb_rhdr_t		phs_header;
	uint32_t		phs_type;
	hal_state_t		phs_state;
} pdk_psdb_hal_state_t;

/* hs_state */
#define HS_STATE_NOT_INIT	0
#define HS_STATE_STOPPED	1
#define HS_STATE_RUNNING	2
#define HS_STATE_RELEASED	3

/* hs_fault */
#define HS_FAULT_NONE		0
#define HS_FAULT_INTERNAL	1
#define HS_FAULT_EXTERNAL	2

/* hs_link_cfg & hs_link_state */
#define HS_LINK_AUTO		0
#define HS_LINK_DOWN		0
#define HS_LINK_TP		1
#define HS_LINK_BNC		2
#define HS_LINK_AUI		3
#define HS_LINK_TPFD		4
#define HS_LINK_TX		5
#define HS_LINK_TXFD		6
#define HS_LINK_T4		7

typedef struct hal_speedometer_s {
	uint32_t	hsp_version;
#define HSP_VERSION 1
	uint32_t	hsp_state;
	uint32_t	hsp_poll_rate;							/* in ms */
	uint32_t	notification_type;
#define PSDB_GREATER_THAN_OR_EQUAL	1
#define PSDB_LESS_THAN			2
/*
	pps - packets per second
	ips - interrupts per second
	eps - errors per second
	pctutil - percent utilization
	hwm - high water mark
*/
	uint32_t	hsp_tx_pps;
	uint32_t	hsp_tx_pps_hwm;
	uint32_t	hsp_tx_ips;
	uint32_t	hsp_tx_ips_hwm;
	uint32_t	hsp_tx_pctutil;
	uint32_t	hsp_tx_pctutil_hwm;
	uint32_t	hsp_tx_eps;
	uint32_t	hsp_tx_eps_hwm;
	uint32_t	hsp_rx_pps;
	uint32_t	hsp_rx_pps_hwm;
	uint32_t	hsp_rx_ips;
	uint32_t	hsp_rx_ips_hwm;
	uint32_t	hsp_rx_pctutil;
	uint32_t	hsp_rx_pctutil_hwm;
	uint32_t	hsp_rx_eps;
	uint32_t	hsp_rx_eps_hwm;
} hal_speedometer_t;

#define PSDB_TYPE_HAL_SPEEDOMETER_GET PSDB_RECORD_TYPE(PSDB_CLASS_PDK_HAL, 2)

typedef struct {
	pdk_psdb_rhdr_t		phsg_header;
	hal_speedometer_t	phsg_speed;
} pdk_psdb_hal_speedometer_get_t;

#define PSDB_TYPE_HAL_SPEEDOMETER_NOTIFICATION PSDB_RECORD_TYPE(PSDB_CLASS_PDK_HAL, 3)
typedef struct {
	pdk_psdb_rhdr_t		phsn_header;
	hal_speedometer_t	phsn_speed;
} pdk_psdb_hal_speedometer_notification_t;

#define PSDB_TYPE_HAL_SPEEDOMETER_SET_PARAMS PSDB_RECORD_TYPE(PSDB_CLASS_PDK_HAL, 4)
typedef struct {
	pdk_psdb_rhdr_t		phss_header;
	hal_speedometer_t	phss_speed;
} pdk_psdb_hal_speedometer_set_params_t;

#define PSDB_TYPE_HAL_FAULT_NOTIFICATION_SET PSDB_RECORD_TYPE(PSDB_CLASS_PDK_HAL, 5)
#define PSDB_TYPE_HAL_FAULT_NOTIFICATION_CLEAR PSDB_RECORD_TYPE(PSDB_CLASS_PDK_HAL, 6)
#define PSDB_TYPE_HAL_FAULT_NOTIFICATION PSDB_RECORD_TYPE(PSDB_CLASS_PDK_HAL, 7)
typedef struct {
	pdk_psdb_rhdr_t	phf_header;
	uint32_t	phf_internal_fault;  /* 0 indicates external fault */
	uint32_t	phf_on;
} pdk_psdb_hal_fault_t;


/******************************* PAL layer *******************************/
#define PSDB_TYPE_PAL_SET_MODE PSDB_RECORD_TYPE(PSDB_CLASS_PDK_PAL, 1)
typedef struct {
	pdk_psdb_rhdr_t	ppsm_header;
	uint32_t	ppsm_mode;
	uint32_t	ppsm_signal;
} pdk_psdb_pal_set_mode_t;

/* PDK_PAL_MODEs */
#define PAL_NOTIFICATION_SIGNAL_MODE 0x1
#define PAL_NOTIFICATION_MODE 0x2

#define PSDB_TYPE_PAL_CLEAR_MODE PSDB_RECORD_TYPE(PSDB_CLASS_PDK_PAL, 2)
typedef struct {
	pdk_psdb_rhdr_t	ppcm_header;
	uint32_t	ppcm_mode;
} pdk_psdb_pal_clear_mode_t;

#define PSDB_TYPE_PAL_GET_HW_INFO PSDB_RECORD_TYPE(PSDB_CLASS_PDK_PAL, 3)
typedef struct {
	uint32_t hi_version;
#define HI_VERSION 2
	int32_t	 hi_rmapi_version;
#define HI_RMAPI_VERSION 030100
/* HI_RMAPI_VERSION:  MMmmff MM: Major version mm: minor version ff: bug fix */
	int32_t  hi_state;
	uint32_t hi_sft_version;
	int32_t  hi_bus;
	int32_t  hi_devno;
	uint32_t hi_adapter_id;
	uint32_t hi_chipid;	/* Device ID and Vendor ID */
	uint32_t hi_cfgcs;	/* Status and Command */
	uint32_t hi_revinfo;	/* Class Code and Revision ID */
	uint32_t hi_ltcls;	/* BIST/Header Type/Latency Timer/Cache Line Size */
	uint32_t hi_bar[6];	/* Base Address Registers */
	uint32_t hi_ssid;	/* Subsystem ID and Subsystem Vendor ID */
	uint32_t hi_rombar;	/* Expansion ROM Base Address */
	uint32_t hi_irq;	/* Max_Lat/Min_Gnt/Interrupt Pin/Interrupt Line */
	uint32_t hi_macaddr[3];	/* Factory Address and Current Address */
	/* FA[0]:FA[1]:FA[2]:FA[3],FA[4]:FA[5]:CA[0]:CA[1],CA[2]:CA[3]:CA[4]:CA[5] */
} pal_hw_info_t;

/* hi state */
#define HI_STATE_NOT_INITED -1
#define HI_STATE_EXISTS	0
typedef struct {
	pdk_psdb_rhdr_t pps_header;
	pal_hw_info_t     pps_info;
} pdk_psdb_pal_get_hw_info_t;

/*****************************  Read Flag ******************************/

#define PSDB_READ_FLAG(record_type) ( \
	((record_type) == PSDB_TYPE_CONFIGURATION) ? 1 : \
	((record_type) == PSDB_TYPE_HAL_GET_STATE) ? 1 : \
	((record_type) == PSDB_TYPE_HAL_SPEEDOMETER_GET) ? 1 : \
	((record_type) == PSDB_TYPE_PAL_GET_HW_INFO) ? 1 : \
	0 )


/*******************************  COMMON *******************************/
/* Global PDK control call */
#define PDK_PSDB 1000
#define PDK_PSDB_INLINE 1001

/* Test Codes */
#define PSDB_TEST_SUCCESS	1
#define PSDB_TEST_ERROR		2
#define PSDB_TEST_ASYNC		3

/* Error Codes */
#define PSDB_PROTOCOL_ERROR 2000
#define PSDB_BAD_PARAM 2001
#define PSDB_INCORRECT_SIZE 2002
#define PSDB_BAD_DEVICE_ID 2003
#define PSDB_TRUNK_IN_USE 2004
#define PSDB_FAILOVER_IN_USE 2005
#define PSDB_PPA_OUT_RANGE 2006
#define PSDB_LAYER_PUSH_ERROR 2007
#define PSDB_SYSTEM_ERROR 2008

#endif PDK_PSDB_H

