/*==========================================================================
**
**	psdb_fio.c
**
**	PSDB file IO library functions
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	ram	01/12/99 v1.0	Created
**
**==========================================================================
**
**	Notes:
**==========================================================================
*/

#include <stdio.h>
#include <pdk_port.h>
#undef _KERNEL
#include <pdk_global.h>
#include <pdk_types.h>

#include <sys/stat.h>
#include <fcntl.h>

#include <pdk_psdb.h>

#include <stropts.h>
#include <sys/conf.h>
#include <psdb_io.h>
#include <rmapi.h>
#include <errno.h>

extern int32_t rmapi_debug;
extern int errno;

#define DEBUG1 if (rmapi_debug>0) printf
#define DEBUG2 if (rmapi_debug>1) printf
#define DEBUG3 if (rmapi_debug>2) printf
#define DEBUG4 if (rmapi_debug>3) printf

rmapi_fd_t
psdb_fio_open(dev, flags)
char *dev;
uint32_t flags;
{
	int32_t fd;

	if (dev == 0)
		fd = open("/dev/zrm", O_RDWR);
	else
		fd = open(dev, O_RDWR);
		

	if (fd == -1)
		return(-1);

	while(ioctl(fd, I_POP, 0) >= 0)
		; /* Pop all modules which may have been autopushed */

	return(fd);
}

uint32_t
psdb_fio_close(fd)
rmapi_fd_t fd;
{
	if (fd == -1)
		return(0);

 	return(close(fd));
}

uint32_t
psdb_fio_send(fd, prp, read_flag)
rmapi_fd_t fd;
pdk_psdb_rhdr_t *prp;
int32_t read_flag;
{
	pdk_psdb_error_t pe;
	uint32_t dev_id;
	int32_t size;
	int32_t read_size;

	if (fd == -1)
		return(0);

	size = psdb_pton(prp->ppr_header.Size);
	dev_id = psdb_pton(prp->ppr_dev_id);

	DEBUG3("psdb_fio_send(%d): send record type 0x%x of %d bytes\n", dev_id, psdb_pton(prp->ppr_header.Type), size);
	if (rmapi_debug > 2) {
		int32_t i;
		uint32_t *lp;

		lp = (uint32_t *) prp;
		for (i=0;i<(size/4);i++, lp++) {
			DEBUG3("%8x ", *lp);
			if (i >= 15)
				break;

			if ((i%4) == 3)
				DEBUG3("\n");
		}
		DEBUG3("\n");
	}
	write(fd, prp, size);	/* OS dependent */

	read_size = read(fd, &pe, sizeof(pe));
	DEBUG3("psdb_fio_send(%d): response record type 0x%x received size %d\n", dev_id, psdb_pton(pe.pe_header.ppr_header.Type), read_size);
	if (rmapi_debug > 2) {
		int32_t i;
		uint32_t *lp;

		lp = (uint32_t *) &pe;
		for (i=0;i<(sizeof(pe)/4);i++, lp++) {
			DEBUG3("%8x ", *lp);
			if (i >= 15)
				break;

			if ((i%4) == 3)
				DEBUG3("\n");
		}
		DEBUG3("\n");
	}

	if (read_size != sizeof(pe))
		return(PSDB_PROTOCOL_ERROR);

	if (pe.pe_header.ppr_header.Type == psdb_ntop(PSDB_TYPE_ERROR)) {
		DEBUG3("psdb_fio_send(%d): Error response %d\n", dev_id, psdb_pton(pe.pe_error));
		if (pe.pe_error_type != prp->ppr_header.Type)
			return(PSDB_PROTOCOL_ERROR);

		return(psdb_pton(pe.pe_error));
	}
	DEBUG3("psdb_fio_send(%d): No error\n", dev_id);

	if (read_flag) {
		read_size = read(fd, prp, size);
		if (read_size != size)
			return(PSDB_PROTOCOL_ERROR);
	}

	return(0);
}

uint32_t
psdb_fio_get(fd, ptr, len)
rmapi_fd_t fd;
void *ptr;
uint32_t len;
{
	PSDB_RHDR *pprp;
	int32_t record_size;

	DEBUG3("psdb_fio_get: get record of %d bytes\n", len);

	if (fd == -1)
		return(0);

	pprp = (PSDB_RHDR *) ptr;

	while ((read(fd, (void *) pprp, sizeof(PSDB_RHDR)) == -1) &&
		(errno == EINTR))
		;  /* Empty loop */

	record_size = psdb_pton(pprp->Size);
	if (len < record_size) {
		DEBUG3("psdb_fio_get: INCORRECT_SIZE\n");
		return(RMAPI_INCORRECT_SIZE);
	}

	ptr = (void *) ((uchar *) ptr + sizeof(PSDB_RHDR));

	if (read(fd, ptr, record_size-sizeof(PSDB_RHDR)) != (record_size-sizeof(PSDB_RHDR))) {
		DEBUG3("psdb_fio_get: INCORRECT_SIZE\n");
		return(RMAPI_INCORRECT_SIZE);
	}
		

	if (rmapi_debug > 2) {
		int32_t i;
		uint32_t *lp;

		DEBUG3("psdb_fio_get: Type 0x%x received\n", psdb_pton(pprp->Type));

		lp = (uint32_t *) ptr;
		for (i=0;i<record_size;i+=4, lp++) {
			DEBUG3("%8x ", *lp);
			if (i >= 15)
				break;

			if ((i%4) == 3)
				DEBUG3("\n");
		}
		DEBUG3("\n");
	}

	return(0);
}

