/*==========================================================================
**
**	psdb_io.c
**
**	PSDB IO library functions
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	ram	01/12/99 v1.0	Created
**
**==========================================================================
**
**	Notes:
**==========================================================================
*/

#include <stdio.h>
#include <pdk_port.h>
#undef _KERNEL
#include <pdk_global.h>
#include <pdk_types.h>
#include <pdk_psdb.h>
#include <psdb_io.h>
#include <psdb_fio.h>
#include <psdb_sio.h>
#include <rmapi.h>

typedef struct psdb_io_file_s {
	rmapi_fd_t (*psdb_io_open)(char *dev, uint32_t flags);
	uint32_t (*psdb_io_close)(rmapi_fd_t fd);
	uint32_t (*psdb_io_send)(rmapi_fd_t fd, pdk_psdb_rhdr_t *prp, int32_t read_flag);
	uint32_t (*psdb_io_get)(rmapi_fd_t fd, void *ptr, uint32_t len);
	rmapi_fd_t psdb_fd;
} psdb_io_file_t;

extern int32_t rmapi_debug;

#define DEBUG1 if (rmapi_debug>0) printf
#define DEBUG2 if (rmapi_debug>1) printf
#define DEBUG3 if (rmapi_debug>2) printf
#define DEBUG4 if (rmapi_debug>3) printf

psdb_io_file_t psdb_io_file_table[PSDB_IO_NUM_FD];

psdb_io_fd(rfd)
rmapi_fd_t rfd;
{
	if ((rfd < 0) || (rfd >= PSDB_IO_NUM_FD))
		return(-1);

	return(psdb_io_file_table[rfd].psdb_fd);
}


static int first_open = 1;

rmapi_fd_t
psdb_io_open(dev, flags)
char *dev;
uint32_t flags;
{
	int32_t fd;
	psdb_io_file_t *fp;

	if (first_open) {
		first_open = 0;
		for (fd=0; fd<PSDB_IO_NUM_FD; fd++)
			psdb_io_file_table[fd].psdb_fd = -1;
	}

	for (fd=0; fd<PSDB_IO_NUM_FD; fd++)
		if (psdb_io_file_table[fd].psdb_fd == -1)
			break;

	if (fd == PSDB_IO_NUM_FD)
		return(-1);

	fp = &psdb_io_file_table[fd];

	if (dev == 0) {
		fp->psdb_io_open = psdb_fio_open;
		fp->psdb_io_close = psdb_fio_close;
		fp->psdb_io_send = psdb_fio_send;
		fp->psdb_io_get = psdb_fio_get;
	}
	else if (dev[0] == '/') {
		fp->psdb_io_open = psdb_fio_open;
		fp->psdb_io_close = psdb_fio_close;
		fp->psdb_io_send = psdb_fio_send;
		fp->psdb_io_get = psdb_fio_get;
	}
	else {
		fp->psdb_io_open = psdb_sio_open;
		fp->psdb_io_close = psdb_sio_close;
		fp->psdb_io_send = psdb_sio_send;
		fp->psdb_io_get = psdb_sio_get;
	}

	fp->psdb_fd = fp->psdb_io_open(dev, flags);
	if (fp->psdb_fd == -1)
		return(-1);

	return(fd);
}

uint32_t
psdb_io_close(fd)
rmapi_fd_t fd;
{
	psdb_io_file_t *fp;
	uint32_t err;

	if ((fd < 0) || (fd >= PSDB_IO_NUM_FD))
		return(-1);

	fp = &psdb_io_file_table[fd];

	if (fp->psdb_fd == -1)
		return(-1);

	err = fp->psdb_io_close(fp->psdb_fd);

	fp->psdb_fd = -1;

	return(err);
}

uint32_t
psdb_io_send(fd, prp, read_flag)
rmapi_fd_t fd;
pdk_psdb_rhdr_t *prp;
int32_t read_flag;
{
	psdb_io_file_t *fp;
	pdk_psdb_error_t pe;
	uint32_t dev_id;
	int32_t size;
	int32_t read_size;

	if ((fd < 0) || (fd >= PSDB_IO_NUM_FD))
		return(-1);

	fp = &psdb_io_file_table[fd];
	if (fp->psdb_fd == -1)
		return(-1);

	return(fp->psdb_io_send(fp->psdb_fd, prp, read_flag));
}

uint32_t
psdb_io_get(fd, ptr, len)
rmapi_fd_t fd;
void *ptr;
uint32_t len;
{
	psdb_io_file_t *fp;
	PSDB_RHDR *pprp;
	int32_t record_size;

	if ((fd < 0) || (fd >= PSDB_IO_NUM_FD))
		return(-1);

	fp = &psdb_io_file_table[fd];
	if (fp->psdb_fd == -1)
		return(-1);

	return(fp->psdb_io_get(fp->psdb_fd, ptr, len));
}

