/*==========================================================================
**
**	psdb_sio.c
**
**	PSDB Socket IO library functions
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	ram	11/04/99 v1.0	Created
**
**==========================================================================
**
**	Notes:
**==========================================================================
*/

#include <stdio.h>
#include <pdk_port.h>
#undef _KERNEL
#include <pdk_global.h>
#include <pdk_types.h>

#include <pdk_psdb.h>

#include <socket.h>
#include <netinet/in.h>
#include <psdb_io.h>
#include <psdb_sio.h>
#include <rmapi.h>
#include <netdb.h>

extern int32_t rmapi_debug;

#define DEBUG1 if (rmapi_debug>0) printf
#define DEBUG2 if (rmapi_debug>1) printf
#define DEBUG3 if (rmapi_debug>2) printf
#define DEBUG4 if (rmapi_debug>3) printf

/*
 * Either dev_id or ppa should be valid, but not both.  The one which is not
 * valid should be set to -1.  On OS's which support dev_id's which are
 * different than ppa values, the dev_id or the ppa may be used.  Otherwise
 * it makes no difference which is used.
 */

static int first_open = 1;
int psdb_open_fds_index = 0;
rmapi_fd_t psdb_open_fds[PSDB_IO_NUM_FD];
void psdb_atexit();

rmapi_fd_t
psdb_sio_open(dev, flags)
char *dev;
uint32_t flags;
{
	int32_t fd;
	struct sockaddr_in sin;
	pdk_psdb_io_open_t pioo;
	struct hostent *hp;

	fd = socket(AF_INET, SOCK_STREAM, 0);

	if (fd < 0) {
		fprintf(stderr, "psdb_sio_open:  Could not create TCP socket on port 7000\n");
		exit(0);
	}

	memset(&sin, 0, sizeof(struct sockaddr_in));

	hp = gethostbyname(dev);
	if (hp == 0) {
		fprintf(stderr, "psdb_sio_open:  Could not find host %s\n", dev);
		exit(0);
	}

	sin.sin_family = AF_INET;
	sin.sin_port = htons(7000);
	sin.sin_addr.s_addr = (*((ulong *) hp->h_addr)); /* loopback */
	if (connect(fd, (struct sockaddr *) &sin, sizeof(struct sockaddr_in)) != 0) {
		close(fd);
		fprintf(stderr, "psdb_sio_open:  Could not connect on TCP socket for host %s\n", dev);
		return(-1);
	}

	if (first_open) {
		first_open = 0;
		atexit(psdb_atexit);
	}

	psdb_open_fds[psdb_open_fds_index++] = fd;

	pioo.pioo_header.ppr_header.Size = psdb_ntop(sizeof(pdk_psdb_io_open_t));
	pioo.pioo_header.ppr_header.Type = psdb_ntop(PSDB_TYPE_IO_OPEN);
	pioo.pioo_header.ppr_dev_id = 0; /* Unused */
	pioo.pioo_flags = 0;

	psdb_sio_send(fd, &pioo.pioo_header, 0);
	return(fd);
}

uint32_t
psdb_sio_close(fd)
rmapi_fd_t fd;
{
	int from, to;
	pdk_psdb_io_close_t pioc;

	for (from=0, to=0; from<psdb_open_fds_index; from++) {
		if (fd != psdb_open_fds[from])
			psdb_open_fds[to++] = psdb_open_fds[from];
	}

	psdb_open_fds_index = to;

	pioc.pioc_header.ppr_header.Size = psdb_ntop(sizeof(pdk_psdb_io_close_t));
	pioc.pioc_header.ppr_header.Type = psdb_ntop(PSDB_TYPE_IO_CLOSE);
	pioc.pioc_header.ppr_dev_id = 0; /* Unused */

	psdb_sio_send(fd, &pioc.pioc_header, 0);

#ifndef SHUT_RDWR
#define SHUT_RDWR 2
#endif

	shutdown(fd, SHUT_RDWR);
	close(fd);
}

void
psdb_atexit()
{
	while (psdb_open_fds_index > 0) {
		psdb_sio_close(psdb_open_fds[0]);
	}
}

uint32_t
psdb_sio_send(fd, prp, read_flag)
rmapi_fd_t fd;
pdk_psdb_rhdr_t *prp;
int32_t read_flag;
{
	pdk_psdb_error_t pe;
	uint32_t dev_id;
	int32_t size;
	int32_t read_size;

	size = psdb_pton(prp->ppr_header.Size);
	dev_id = psdb_pton(prp->ppr_dev_id);

	DEBUG3("psdb_sio_send(%d): send record type 0x%x of %d bytes\n", dev_id, psdb_pton(prp->ppr_header.Type), size);
	if (rmapi_debug > 2) {
		int32_t i;
		uint32_t *lp;

		lp = (uint32_t *) prp;
		for (i=0;i<(size/4);i++, lp++) {
			DEBUG3("%8x ", *lp);
			if (i >= 15)
				break;

			if ((i%4) == 3)
				DEBUG3("\n");
		}
		DEBUG3("\n");
	}
	write(fd, prp, size);	/* OS dependent */

	read_size = read(fd, &pe, sizeof(pe));
	DEBUG3("psdb_sio_send(%d): response record type 0x%x received size %d\n", dev_id, psdb_pton(pe.pe_header.ppr_header.Type), read_size);
	if (rmapi_debug > 2) {
		int32_t i;
		uint32_t *lp;

		lp = (uint32_t *) &pe;
		for (i=0;i<(sizeof(pe)/4);i++, lp++) {
			DEBUG3("%8x ", *lp);
			if (i >= 15)
				break;

			if ((i%4) == 3)
				DEBUG3("\n");
		}
		DEBUG3("\n");
	}

	if (read_size != sizeof(pe))
		return(PSDB_PROTOCOL_ERROR);

	if (pe.pe_header.ppr_header.Type == psdb_ntop(PSDB_TYPE_ERROR)) {
		DEBUG3("psdb_sio_send(%d): Error response %d\n", dev_id, psdb_pton(pe.pe_error));
		if (pe.pe_error_type != prp->ppr_header.Type)
			return(PSDB_PROTOCOL_ERROR);

		return(psdb_pton(pe.pe_error));
	}
	DEBUG3("psdb_sio_send(%d): No error\n", dev_id);

	if (read_flag) {
		read_size = read(fd, prp, size);
		if (read_size != size)
			return(PSDB_PROTOCOL_ERROR);
	}

	return(0);
}

uint32_t
psdb_sio_get(fd, ptr, len)
rmapi_fd_t fd;
void *ptr;
uint32_t len;
{
	PSDB_RHDR *pprp;
	pdk_psdb_io_get_t piog;
	int32_t record_size;
	int32_t err;

	piog.piog_header.ppr_header.Size = psdb_ntop(sizeof(pdk_psdb_io_open_t));
	piog.piog_header.ppr_header.Type = psdb_ntop(PSDB_TYPE_IO_GET);
	piog.piog_header.ppr_dev_id = 0; /* Unused */
	piog.piog_len = psdb_ntop(len);

	if ((err = psdb_sio_send(fd, &piog.piog_header, 0)) != 0)
		return(err);

	pprp = (PSDB_RHDR *) ptr;

	read(fd, (void *) pprp, sizeof(PSDB_RHDR));

	record_size = psdb_pton(pprp->Size);
	if (len < record_size)
		return(RMAPI_INCORRECT_SIZE);

	ptr = (void *) ((uchar *) ptr + sizeof(PSDB_RHDR));

	read(fd, ptr, record_size-sizeof(PSDB_RHDR));

	return(0);
}

