/*==========================================================================
**
**	./pdk/src/drivers/pal/rlalarm.c
**
**	RAINlink switch application
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	ram	01/25/99 v1.0	Created
**
**==========================================================================
**
**	Notes:
**==========================================================================
*/

#include <stdio.h>
#include <pdk_port.h>
#undef _KERNEL
#include <pdk_global.h>
#include <pdk_types.h>

#include <pdk_errno.h>

#include <pdk_queue.h>
#include <rmapi.h>
#include <signal.h>

int32_t rl_debug = 0;

#define DEBUG1 if (rl_debug>0) printf
#define DEBUG2 if (rl_debug>1) printf
#define DEBUG3 if (rl_debug>2) printf
#define DEBUG4 if (rl_debug>3) printf

extern int32_t rmapi_debug;


extern char *optarg;
extern int optind;
int rmapi_fd;
int rmapi_notification_fd;
FILE *flist = 0;

void rc_wait_for_notification(rmapi_general_notification_t *p);
void rl_precedence(int32_t rs_dev_id, int32_t *rs_prec_list, int i);
void rl_clear_hal_notifications(int32_t ppa);
void rc_closedown(int sig);
int32_t rs_dev_id;

main(argc, argv)
int	argc;
char	*argv[];
{
	rmapi_set_mode_t rsm;
	rmapi_pal_set_mode_t psm;
	hal_speedometer_t *hsp;
	int32_t rs_to_ppa;
	int32_t rs_prec_list[PSDB_MAX_PORTS];
	int32_t err;
	char c;
	int i;
	rmapi_general_notification_t gn={0};
	rmapi_hal_speedometer_set_params_t hset;
	char hostname_space[128];
	char *hostname;
	int32_t run_switch = 0;
	int32_t run_hal_get = 0;
	int32_t run_rlk_get = 0;
	int32_t run_precedence = 0;

	if (argc < 2) {
		usage("Must specify a flag");
		return(-1);
	}

	hostname = 0;

	while ((c = getopt(argc, argv, "h:d:s:m:r:p:")) != -1) {
		switch (c) {
		case 'h':
			strncpy(hostname_space, optarg, 128);
			hostname = hostname_space;
			break;
		case 'd':
			rl_debug = atoi(optarg);
			rmapi_debug = rl_debug;
			break;

		case 's':
			if (run_precedence || run_hal_get || run_rlk_get) {
				usage("-s cannot be used with -m, -r or -p\n");
				exit(1);
			}
			if (isdigit(*optarg) == 0) {
				usage("-s requires two integer parameters");
				exit(1);
			}
			if ((argv[optind] == 0) || (isdigit(*argv[optind]) == 0)) {
				usage("-s requires two integer parameters");
				exit(1);
			}

			rs_dev_id = atoi(optarg);
			rs_to_ppa = atoi(argv[optind++]);
			if (rs_to_ppa < 0 || rs_to_ppa > PSDB_MAX_PORTS) {
				usage("to_ppa is out of range");
				exit(1);
			}
			run_switch = 1;
			break;

		case 'm':
			if (run_precedence || run_switch) {
				usage("-m cannot be used with -s or -p\n");
				exit(1);
			}
			if (isdigit(*optarg) == 0) {
				usage("-m requires an integer parameters");
				exit(1);
			}

			rs_dev_id = atoi(optarg);
			if (rs_dev_id < 0 || rs_dev_id > PSDB_MAX_PORTS) {
				usage("to_ppa is out of range");
				exit(1);
			}

			run_hal_get = 1;
			break;

		case 'r':
			if (run_precedence || run_switch) {
				usage("-r cannot be used with -s or -p\n");
				exit(1);
			}
			if (isdigit(*optarg) == 0) {
				usage("-r requires an integer parameters");
				exit(1);
			}

			rs_dev_id = atoi(optarg);
			if (rs_dev_id < 0 || rs_dev_id > PSDB_MAX_PORTS) {
				usage("to_ppa is out of range");
				exit(1);
			}

			run_rlk_get = 1;
			break;

		case 'p':
			if (run_switch || run_hal_get || run_rlk_get) {
				usage("-p cannot be used with -m, -r or -s\n");
				exit(1);
			}


			rs_dev_id = atoi(optarg);
			if (rs_dev_id < 0 || rs_dev_id > PSDB_MAX_PORTS) {
				usage("to_ppa is out of range");
				exit(1);
			}

			for (i=0;optind<argc;optind++, i++)  {
				rs_prec_list[i] = atoi(argv[optind]);
			}

			if (i == 0) {
				usage("Must provide ppa list");
				exit(1);
			}

			run_precedence = 1;
			break;

		default:
			usage("");
			exit(1);
		}
	}
	if ((rmapi_fd = rmapi_open(hostname, 0)) == -1) {
		perror("Open of /dev/zrm failed");
		exit(1);
	}

	if (run_switch) {
		rlswitch(rs_dev_id, rs_to_ppa);
		rmapi_close(rmapi_fd);
		exit(0);
	}

	if (run_precedence || run_rlk_get) {
		rsm.rsm_dev_id = rs_dev_id;
		rsm.rsm_mode = RLK_NOTIFICATION_MODE;
		rsm.rsm_time = 0;
		rmapi_ctl(rmapi_fd, RMAPI_CTL_SET_MODE, &rsm);
	}

	if (run_hal_get) {
		hset.rhss_ppa = rs_dev_id;
		hsp = &hset.rhss_speed;
		hsp->hsp_poll_rate=10000; /* in milliseconds */
		hsp->hsp_version = HSP_VERSION;
		hsp->hsp_rx_pctutil=20;
		hsp->notification_type=RMAPI_GREATER_THAN_OR_EQUAL;

		err = rmapi_ctl(rmapi_fd, RMAPI_CTL_HAL_SPEEDOMETER_SET_PARAMS, &hset);
		if (err != 0) {
			fprintf(stderr, "Error %d returned from speedometer notificaion setting\n", err);
			rmapi_close(rmapi_fd);
			exit(1);
		}
#ifdef RMAPI_CTL_HAL_FAULT_NOTIFICATION_SET
		{
			rmapi_hal_fault_t rhf;

			rhf.rhf_ppa = rs_dev_id;
			rhf.rhf_internal_fault = 0; /* Unused */
			rhf.rhf_on = 0; /* Unused */
			err = rmapi_ctl(rmapi_fd, RMAPI_CTL_HAL_FAULT_NOTIFICATION_SET, &rhf);
			if (err != 0) {
				rl_clear_hal_notifications(rs_dev_id);
				fprintf(stderr, "Error %d returned from fault notificaion setting\n", err);
				rmapi_close(rmapi_fd);
				exit(1);
			}
		}
#endif
	}

	psm.rpsm_dev_id = rs_dev_id;
	psm.rpsm_mode = PAL_NOTIFICATION_MODE;
	psm.rpsm_signal = 0;
	rmapi_ctl(rmapi_fd, RMAPI_CTL_PAL_SET_MODE, &psm);

	if (run_precedence) {
		rmapi_notification_fd = rmapi_fd;

		if ((rmapi_fd = rmapi_open(hostname, 0)) == -1) {
			perror("Open of /dev/zrm failed");
			rmapi_close(rmapi_notification_fd);
			exit(1);
		}

		if (rl_check_precedence(rs_dev_id, rs_prec_list, i) != 0) {
			usage("Precedence list contains ppa's which are not part of the failover group");
			rmapi_close(rmapi_notification_fd);
			rmapi_close(rmapi_fd);
			exit(1);
		}

		/* Never returns */
		rl_precedence(rs_dev_id, rs_prec_list, i);

		/* Never returns ... but */
		rmapi_close(rmapi_fd);
		rmapi_fd = rmapi_notification_fd;
	}

	if (run_hal_get || run_rlk_get) {
		(void) signal(SIGHUP, rc_closedown);
		(void) signal(SIGTERM, rc_closedown);
		(void) signal(SIGINT, rc_closedown);
		(void) signal(SIGQUIT, rc_closedown);
		rc_wait_for_notification(&gn);
		rl_clear_hal_notifications(rs_dev_id);
	}

	rmapi_close(rmapi_fd);
}

void
rc_wait_for_notification(gnp)
rmapi_general_notification_t *gnp;
{
	int i=10;
	hal_speedometer_t *hsp;

	hsp = &gnp->rh.rhsn_speed;

	while (i--) {
		rmapi_get(rmapi_fd, gnp, sizeof(rmapi_general_notification_t));

		switch(gnp->rfn.rfn_header.Type) {
			case RMAPI_CTL_FAILOVER_NOTIFICATION:
				printf("FAILOVER from ppa %d to %d\n", 
				gnp->rfn.rfn_from_ppa, gnp->rfn.rfn_to_ppa);
				break;

			case RMAPI_CTL_FAULT_NOTIFICATION:
				printf("%s FAULT for ppa %d is %s\n", 
				gnp->rftn.rftn_internal_fault ? "INTERNAL" : "EXTERNAL", 
				gnp->rftn.rftn_fault_ppa, gnp->rftn.rftn_on ? "on" : "off");
				break;

			case RMAPI_CTL_HAL_SPEEDOMETER_NOTIFICATION: {
				printf("Returned from speedometer notificaion OK\n");
				printf("Receive  percent utilization:		%2d\n", hsp->hsp_rx_pctutil);
				}						
				break;
#ifdef RMAPI_CTL_HAL_FAULT_NOTIFICATION_SET
			case RMAPI_CTL_HAL_FAULT_NOTIFICATION: {
				rmapi_hal_fault_t *rhfp;

				rhfp = &gnp->rhf;

				printf("%s fault %s for ppa %d\n", rhfp->rhf_internal_fault ? "Internal" : "External", rhfp->rhf_on ? "on" : "off", rhfp->rhf_ppa);
				break;
			}
#endif

			default:
				printf("Received unknown notification - 0x%x\n", gnp->rfn.rfn_header.Type); 
		}
	}
}

rlswitch(dev_id, to_ppa)
int32_t dev_id;
int32_t to_ppa;
{
	rmapi_failover_t rf;
	int32_t err;

	if (dev_id < 0 || dev_id > PSDB_MAX_PORTS) {
		usage("Device id is out of range");
		exit(1);
	}

	rf.rf_dev_id = dev_id;
	rf.rf_to_ppa = to_ppa;
	err = rmapi_ctl(rmapi_fd, RMAPI_CTL_FAILOVER, &rf);

	if (err == 0)
		exit(0);  /* Success */

	switch (err) {
		case RMAPI_PROTOCOL_ERROR:
			fprintf(stderr, "Internal RMAPI protocol error\n");
			break;

		case RMAPI_BAD_PARAM:
		case RMAPI_PPA_OUT_RANGE:
		case RMAPI_LAYER_PUSH_ERROR:
			fprintf(stderr, "To PPA %d is not in a failover group\n", to_ppa);
			break;

		case RMAPI_BAD_DEVICE_ID:
			fprintf(stderr, "Device id %d is not a failover group\n", dev_id);
			break;

		default:
			fprintf(stderr, "Error %d returned from RAINlink layer\n", err);
	}
}

rl_check_precedence(dev_id, rs_prec_list, num_ppa)
int32_t dev_id;
int32_t *rs_prec_list;
int num_ppa;
{
	rmapi_configuration_t rc;
	rlk_configuration_t *rcp;
	int32_t desired_ppa;
	int i,j;

	rc.rc_dev_id = dev_id;
	rc.rc_ppa = -1;
	rc.rc_type = 0;

	rc.rc_configuration.rc_offset_ports = 0;
	rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc);
	rcp = &rc.rc_configuration;


	if (num_ppa != rcp->rc_num_primary_ports) {
		printf("Warning(%d):  precedence list(%d) is not the same size as the number of primary ports(%d) in the failover group\n", dev_id, num_ppa, rcp->rc_num_primary_ports);
	}

	for (i=0; i<num_ppa; i++) {
		desired_ppa = rs_prec_list[i];

		for(j=0; j<rcp->rc_num_primary_ports; j++)
			if (desired_ppa == rcp->rc_primary_ports[j])
				break;

		if (j == rcp->rc_num_primary_ports)
			return(-1);
	}

	return(0);
}

void
rl_precedence(dev_id, prec_list, num_ppa)
int32_t dev_id;
int32_t *prec_list;
int num_ppa;
{
	rmapi_general_notification_t gnp={0};

	rl_precedence_switch(dev_id, prec_list, num_ppa);

	while (1) {
		rmapi_get(rmapi_notification_fd, &gnp, sizeof(gnp));

		switch(gnp.rfn.rfn_header.Type) {
			case RMAPI_CTL_FAILOVER_NOTIFICATION:
			case RMAPI_CTL_FAULT_NOTIFICATION:
				rl_precedence_switch(dev_id, prec_list, num_ppa);
				break;

			default:
				printf("Received unknown notification - 0x%x\n", gnp.rfn.rfn_header.Type); 
		}
	}
}

rl_precedence_switch(dev_id, prec_list, num_ppa)
int32_t dev_id;
int32_t *prec_list;
int num_ppa;
{
	rmapi_configuration_t rc, prim_rc;
	rlk_configuration_t *rcp, *prim_rcp;
	int32_t desired_ppa;
	int32_t err;
	int i, j;

	prim_rc.rc_dev_id = dev_id;
	prim_rc.rc_ppa = -1;
	prim_rc.rc_type = 0;
	prim_rc.rc_configuration.rc_offset_ports = 0;
	rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &prim_rc);

	prim_rcp = &prim_rc.rc_configuration;

	for (i=0; i<num_ppa; i++) {
		desired_ppa = prec_list[i];

		if (prim_rcp->rc_failover_port == desired_ppa) {
			rc.rc_dev_id = -1;
			rc.rc_ppa = desired_ppa;
			rc.rc_type = 0;
			rc.rc_configuration.rc_offset_ports = 0;
			rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc);

			rcp = &rc.rc_configuration;

			if (rcp->rc_linkup)
				printf("PORT%d primary\n", desired_ppa);
			else
				printf("LINK down\n");

			return(0);
		}

		for(j=0; j<prim_rcp->rc_num_primary_ports; j++)
			if (desired_ppa == prim_rcp->rc_primary_ports[j])
				break;

		if (j != prim_rcp->rc_num_primary_ports) {
			rc.rc_dev_id = -1;
			rc.rc_ppa = desired_ppa;
			rc.rc_type = 0;
			rc.rc_configuration.rc_offset_ports = 0;
			rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc);

			rcp = &rc.rc_configuration;
			if (rcp->rc_linkup) {
				rmapi_failover_t rf;

				rf.rf_dev_id = dev_id;
				rf.rf_to_ppa = desired_ppa;
				err = rmapi_ctl(rmapi_fd, RMAPI_CTL_FAILOVER, &rf);
				rc.rc_dev_id = -1;
				rc.rc_ppa = desired_ppa;
				rc.rc_type = 0;
				rc.rc_configuration.rc_offset_ports = 0;
				rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc);
				rcp = &rc.rc_configuration;

				if (rcp->rc_linkup == 0) {
					/* link change will cause us to run again */
					printf("PORT%d down - move\n", desired_ppa);
					rf.rf_dev_id = dev_id;
					rf.rf_to_ppa = -1;
					rmapi_ctl(rmapi_fd, RMAPI_CTL_FAILOVER, &rf);
				}
				printf("PORT%d primary\n", desired_ppa);
				return(0);
			}
		}
	}

	printf("LINK down\n");
	return(0);
}

void
rl_clear_hal_notifications(ppa)
int32_t ppa;
{
	rmapi_hal_speedometer_set_params_t hset;
	hal_speedometer_t *hsp;
	uint32_t err;

	hset.rhss_ppa = ppa;
	hsp = &hset.rhss_speed;
	hsp->hsp_poll_rate=0;
	hsp->hsp_version = HSP_VERSION;
	hsp->hsp_rx_pctutil=0;
	hsp->notification_type=0;
	err = rmapi_ctl(rmapi_fd, RMAPI_CTL_HAL_SPEEDOMETER_SET_PARAMS, &hset);

	if (err != 0) {
		fprintf(stderr, "Error %d returned from speedometer notificaion setting\n", err);
		rmapi_close(rmapi_fd);
		exit(1);
	}
#ifdef RMAPI_CTL_HAL_FAULT_NOTIFICATION_SET
	{
		rmapi_hal_fault_t rhf;

		rhf.rhf_ppa = rs_dev_id;
		rhf.rhf_internal_fault = 0; /* Unused */
		rhf.rhf_on = 0; /* Unused */
		err = rmapi_ctl(rmapi_fd, RMAPI_CTL_HAL_FAULT_NOTIFICATION_CLEAR, &rhf);
		if (err != 0) {
			fprintf(stderr, "Error %d returned from fault notificaion setting\n", err);
		}
	}
#endif
}

void
rc_closedown(sig)
int sig;
{
	rl_clear_hal_notifications(rs_dev_id);
	exit(1);
}

usage(s)
char *s;
{
	printf("%s\n", s);
	printf("Usage: rlalarm [-d <debug_level> [-s <dev_id> <ppa>] [-r <dev_id>] [-m <ppa>] [-p <dev_id> <ppa> <ppa> ...] [-h <host_name>\n");
	printf("                -d:  set debug level to <debug_level>\n");
	printf("                -s:  switch device <dev_id> to the specified <ppa>\n");
	printf("                -r:  Display RAINlink notifications from specified <dev_id>\n");
	printf("                -m:  Display MAC level notifications from specified <ppa>\n");
	printf("                -p:  set precedence of device <dev_id> on failover\n");
	printf("                     group to order of <ppa> list.  First in list has\n");
	printf("                     highest precedence\n");
	printf("                -h:  connect to remote host <host_name>\n");
}
