/*==========================================================================
**
**	./pdk/src/drivers/pal/rlconfig.c
**
**	RAINlink configuration application
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	ram	06/18/98 v1.0	Created
**
**==========================================================================
**
**	Notes:
**==========================================================================
*/

#include <stdio.h>
#include <pdk_port.h>
#undef _KERNEL
#include <pdk_global.h>
#include <pdk_types.h>

#include <pdk_errno.h>

#include <pdk_queue.h>

#include <rlconfig.h>
#include <rmapi.h>

int32_t rl_debug = 0;

#define DEBUG1 if (rl_debug>0) printf
#define DEBUG2 if (rl_debug>1) printf
#define DEBUG3 if (rl_debug>2) printf
#define DEBUG4 if (rl_debug>3) printf

ppa_array_t *rmapi_find_group(int32_t ppa, int type);
int32_t rmapi_ppa_to_dev(uint32_t ppa, int rmapi_config);
void rc_show();
void rc_teardown();
void rc_list(int format);
void rc_port_state(int32_t ppa);

ppa_array_t *ppa_arrays = 0;
extern char *optarg;
extern int optind;
extern int lineno;
int rmapi_fd;
extern int rmapi_debug;
FILE *flist = 0;

int32_t max_fetch_location = 0;

main(argc, argv)
int	argc;
char	*argv[];
{
	char c;
	int format = 0;
	int run_yyparse = 1;
	int run_show = 0;
	int run_teardown = 0;
	int run_port_state = 0;
	int run_list = 0;
	int32_t ppa;
	char hostname_space[128];
	char *hostname;

	hostname = 0;

	while ((c = getopt(argc, argv, "h:d:sp:tl:")) != -1) {
		switch (c) {
		case 'h':
			strncpy(hostname_space, optarg, 128);
			hostname = hostname_space;
			break;

		case 'd':
			rl_debug = atoi(optarg);
			rmapi_debug = rl_debug;
			break;

		case 's':
			run_show = 1;
			run_yyparse = 0;
			break;

		case 'p':
			if (optind > argc) {
				usage("Must specify a ppa");
				exit(1);
			}

			if (isdigit(*optarg) == 0) {
				usage("Must specify ppa as a number");
				exit(0);
			}

			ppa = atoi(optarg);

			run_port_state = 1;
			run_yyparse = 0;
			break;

		case 't':
			run_teardown = 1;
			run_yyparse = 0;
			break;

		case 'l':
			if (isdigit(*optarg)) {
				if (*(optarg+1) != 0) {
					usage("Format must be single digit and files must not begin with a numeric character");
					exit(1);
				}
				format = atoi(optarg);
				optarg += 2;
				optind++;
			}

			if (optind > argc) {
				usage("Must specify list file");
				exit(1);
			}

			if (isdigit(*optarg)) {
				usage("Format must be single digits and files must not begin with a numeric character");
				exit(1);
			}

			if ((flist = fopen(optarg, "w")) == 0) {
				fprintf(stderr, "Open of %s failed\n", optarg);
				exit(1);
			}
			run_list = 1;
			run_yyparse = 0;
			break;

		default:
			usage("");
			return(-1);
		}
	}

	if ((rmapi_fd = rmapi_open(hostname, 0)) == -1) {
		DEBUG1("rmapi_open failed\n");
		exit(1);
	}

	if (run_show)
		rc_show();

	if (run_port_state)
		rc_port_state(ppa);

	if (run_teardown)
		rc_teardown();

	if (run_list)
		rc_list(format);

	if (run_yyparse) {
		yyparse();
		rc_process();
	}

	rmapi_close(rmapi_fd);
}


ppa_array_t *
rc_create_ppa_array()
{
	ppa_array_t *pa;

	pa = (ppa_array_t *) calloc(1, sizeof(ppa_array_t));
	if (pa == 0)
		return(0);

	pa->pa_dev_id = -1;

	pdk_q_enq((pdk_q_t **) &ppa_arrays, (pdk_q_t *) pa);
	return(pa);
}

void
rc_destroy_ppa_array(pa)
ppa_array_t *pa;
{
	pdk_q_extract((pdk_q_t **) &ppa_arrays, (pdk_q_t *) pa);
	free(pa);
}

/*
 * Push in an increasing ordered list
 */
rc_push_ppa(pa, ppa)
ppa_array_t *pa;
int32_t ppa;
{
	int i;
	int32_t ppa2;

	DEBUG3("rc_push_ppa(%d, %d)\n", pa->pa_index, ppa);
	if (pa->pa_index >= PSDB_MAX_PORTS)
		return(1);

	for (i=0;i<pa->pa_index && pa->pa_array[i]<ppa;i++) ;
		/* Empty */

	for (;i<pa->pa_index;i++) {
		ppa2 = pa->pa_array[i];
		pa->pa_array[i] = ppa;
		ppa = ppa2;
	}

	pa->pa_array[pa->pa_index++] = ppa;
	return(0);
}

/* Find trunk with the specified dev_id which is not pa */
ppa_array_t *
rc_find_group(pa, dev_id, type)
ppa_array_t *pa;
int32_t dev_id;
int type;
{
	ppa_array_t *pa2;

	pa2 = 0;
	while ((pa2 = (ppa_array_t *) pdk_q_next((pdk_q_t **) &ppa_arrays, (pdk_q_t *) pa2)) != 0) {
		/* Don't check self */
		if (pa2 == pa)
			continue;

		switch(type) {
			case 0:
				break;

			case FIND_TRUNK:
				if (pa2->pa_is_trunk == 0)
					continue;

				break;

			case FIND_FAILOVER:
				if (pa2->pa_is_failover == 0)
					continue;

				break;
		}

		if (pa2->pa_dev_id == dev_id) {
			return(pa2);
		}
	}

	/* See if trunk is already built */
	return(rmapi_find_group(dev_id, type));
}

rc_ppa_in_use(ppa)
int32_t ppa;
{
	int i;
	ppa_array_t *pa;

	pa = 0;
	while ((pa = (ppa_array_t *) pdk_q_next((pdk_q_t **) &ppa_arrays, (pdk_q_t *) pa)) != 0) {
		for (i=0;i<pa->pa_index;i++)
			if (pa->pa_array[i] == ppa)
				return(1);

	}

	return(0);
}

rc_trunk_in_use(dev_id)
int32_t dev_id;
{
	int32_t ppa;
	int i;
	ppa_array_t *pa, *trunk_pa;

	if ((trunk_pa = rc_find_group(0, dev_id, 0)) == 0)
		return(0);

	/* The lowest ppa, which is the first ppa, is used as the identifier
	 * for the trunk being added.
	 */
	ppa = trunk_pa->pa_array[0];

	pa = 0;
	while ((pa = (ppa_array_t *) pdk_q_next((pdk_q_t **) &ppa_arrays, (pdk_q_t *) pa)) != 0) {
		/* Don't check the trunk that is being added.  It clearly will
	 	 * contain the lowest ppa
		 */
		if (pa == trunk_pa)
			continue;

		/* Trunks cannot contain trunks - so don't check */
		if (pa->pa_is_trunk)
			continue;

		if (pa->pa_is_failover == 0)
			continue;

		for (i=0;i<pa->pa_index;i++)
			if (pa->pa_array[i] == ppa)
				return(1);
	}

	return(0);
}

rc_set_mode(dev_id, mode)
int32_t dev_id;
uint32_t mode;
{
	ppa_array_t *trunk_pa;

	if ((trunk_pa = rc_find_group(0, dev_id, 0)) == 0)
		return(0);

	switch(mode) {
		case RLK_LAYER_2_MODE:
		case RLK_LAYER_3_MODE:
		case RLK_ROUND_ROBIN_MODE:
		case RLK_IP_TRUNKING_MODE:
		case RLK_BALANCE_MODE:
			if ((trunk_pa = rc_find_group(0, dev_id, FIND_TRUNK)) == 0) {
				fprintf(stderr, "line %d: ", lineno);
				fprintf(stderr, "mode %x can only be set on trunks - %d is not a trunk\n", mode, dev_id);
				exit(0);
			}
			break;

		case RLK_HUB_MODE:
			if ((trunk_pa = rc_find_group(0, dev_id, FIND_FAILOVER)) == 0) {
				fprintf(stderr, "line %d: ", lineno);
				fprintf(stderr, "mode can only be set on failover groups - %d is not a failover group\n", dev_id);
				exit(0);
			}
			break;

		case RLK_INTEGRITY_MODE:
			break;

	}

	trunk_pa->pa_mode |= mode;
	/* LAYER_3 and LAYER_2 modes are mutually exclusive */
	if (mode == RLK_LAYER_2_MODE)
		trunk_pa->pa_mode &= ~RLK_LAYER_3_MODE;
	else if (mode == RLK_LAYER_3_MODE)
		trunk_pa->pa_mode &= ~RLK_LAYER_2_MODE;

	return(0);
}

rc_set_timer_mode(dev_id, time)
int32_t dev_id;
uint32_t time;
{
	ppa_array_t *trunk_pa;

	if (((trunk_pa = rc_find_group(0, dev_id, FIND_FAILOVER)) == 0) ||
	    (trunk_pa->pa_is_failover == 0)) {

		fprintf(stderr, "line %d: ", lineno);
		fprintf(stderr, "mode can only be set on failover groups - %d is not a failover group\n", dev_id);
		exit(0);
	}

	trunk_pa->pa_rx_timeout = time;
	trunk_pa->pa_mode |= RLK_TIMEOUT_MODE;

	return(0);
}

rc_clear_mode(dev_id, mode)
int32_t dev_id;
uint32_t mode;
{
	ppa_array_t *trunk_pa;

	switch(mode) {
		case RLK_LAYER_2_MODE:
		case RLK_LAYER_3_MODE:
		case RLK_ROUND_ROBIN_MODE:
		case RLK_IP_TRUNKING_MODE:
		case RLK_BALANCE_MODE:
			if ((trunk_pa = rc_find_group(0, dev_id, FIND_TRUNK)) == 0) {
				fprintf(stderr, "line %d: ", lineno);
				fprintf(stderr, "mode %x can only be set on trunks - %d is not a trunk\n", mode, dev_id);
				exit(0);
			}
			break;

		case RLK_TIMEOUT_MODE:
		case RLK_HUB_MODE:
			if ((trunk_pa = rc_find_group(0, dev_id, FIND_FAILOVER)) == 0) {
				fprintf(stderr, "line %d: ", lineno);
				fprintf(stderr, "mode can only be set on failover groups - %d is not a failover group\n", dev_id);
				exit(0);
			}
			break;

		case RLK_INTEGRITY_MODE:
			if ((trunk_pa = rc_find_group(0, dev_id, 0)) == 0)
				return(0);

			break;
	}

	
	trunk_pa->pa_clear_mode = mode;
	/* LAYER_3 and LAYER_2 modes are mutually exclusive */
	if (mode == RLK_LAYER_2_MODE)
		trunk_pa->pa_clear_mode &= ~RLK_LAYER_3_MODE;
	else if (mode == RLK_LAYER_3_MODE)
		trunk_pa->pa_clear_mode &= ~RLK_LAYER_2_MODE;

	DEBUG2("rc_clear_mode(%d): %s 0x%x with mode 0x%x and clear_mode 0x%x\n", trunk_pa->pa_dev_id, trunk_pa->pa_is_trunk ? "TRUNK" : (trunk_pa->pa_is_failover ? "FAILOVER" : ""), trunk_pa, trunk_pa->pa_mode, trunk_pa->pa_clear_mode);

	return(0);
}

ppa_array_t *
rmapi_find_group(dev_id, type)
int32_t dev_id;
int type;
{
	ppa_array_t *pa;
	rmapi_configuration_t rc;
	rlk_configuration_t *rcp;
	int32_t i;

	pa = rc_create_ppa_array();

	if (pa == 0)
		return(0);

	rc.rc_ppa = -1;
	rc.rc_dev_id = dev_id;
	rc.rc_type = 0;
	rc.rc_configuration.rc_offset_ports = 0;
	if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc) != 0) {
		rc_destroy_ppa_array(pa);
		return(0);
	}

	rcp = &rc.rc_configuration;

	if (rcp->rc_ppa == -1) {
		rc_destroy_ppa_array(pa);
		return(0);
	}

	if (rcp->rc_num_primary_ports > 1)
		pa->pa_is_failover = 1;

	if (rcp->rc_num_configured_ports > 1) {
		for (i=0;i<rcp->rc_num_configured_ports;i++)
			rc_push_ppa(pa, rcp->rc_configured_ports[i]);

		pa->pa_is_trunk = 1;
	}

	if (((type == FIND_TRUNK) && pa->pa_is_failover) ||
	   ((type == FIND_FAILOVER) && (pa->pa_is_failover == 0))) {
		rc_destroy_ppa_array(pa);
		return(0);
	}

	pa->pa_dev_id = dev_id;
	pa->pa_is_built = 1;

	return(pa);
}

rmapi_ppa_to_dev(ppa, rmapi_config)
uint32_t ppa;
int rmapi_config;
{
	rmapi_configuration_t rc;
	rlk_configuration_t *rcp;
	uint32_t dev_id;

	for (dev_id=0;dev_id<PSDB_MAX_PORTS;dev_id++) {
		rc.rc_ppa = -1;
		rc.rc_dev_id = dev_id;
		rc.rc_type = rmapi_config;
		rc.rc_configuration.rc_offset_ports = 0;
		if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc) != 0)
			continue;

		rcp = &rc.rc_configuration;

		if (rcp->rc_version != RC_VERSION) {
			fprintf(stderr, "Version(%d) mismatch with RLK layer version(%d) in driver.  Need updated rlconfig program\n", RC_VERSION, rcp->rc_version);
			return;
		}

		if (rmapi_config == RMAPI_CONFIG_FAILOVER_TYPE) {
			if ((rcp->rc_num_primary_ports > 1) &&
				(rcp->rc_failover_ppa == rcp->rc_ppa) &&
				(rcp->rc_ppa == ppa)) {

				return(dev_id);
			}
		}
		else {
			if ((rcp->rc_num_configured_ports > 1) &&
				(rcp->rc_configured_ports[0] == ppa)) {

				return(dev_id);
			}
		}
	}
	return(-1);
}

void
rc_show()
{
	rmapi_configuration_t rc, rc2;
	rlk_configuration_t *rcp, *rcp2;
	int32_t ppa;
	int32_t mode;
	uint32_t dev_id;
	int j;
	int no_rainlink = 1;

	for (dev_id=0;dev_id<PSDB_MAX_PORTS;dev_id++) {
		rc.rc_ppa = -1;
		rc.rc_dev_id = dev_id;
		rc.rc_type = RMAPI_CONFIG_TRUNK_TYPE;
		rc.rc_configuration.rc_offset_ports = 0;
		if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc) != 0)
			continue;


		rcp = &rc.rc_configuration;
		no_rainlink = 0;

		if (rcp->rc_version != RC_VERSION) {
			fprintf(stderr, "Version(%d) mismatch with RLK layer version(%d) in driver.  Need updated rlconfig program\n", RC_VERSION, rcp->rc_version);
			return;
		}

		/* Look for trunks first */
		if (rcp->rc_num_configured_ports < 2)
			continue;

		printf("trunk%d", dev_id);
		if (rcp->rc_mode != RLK_BALANCE_MODE) {
			printf("<");
			if (rcp->rc_mode & RLK_ROUND_ROBIN_MODE)
				printf("ROUND_ROBIN");
			else if (rcp->rc_mode & RLK_IP_TRUNKING_MODE)
				printf("IP_TRUNKING");
			else if (rcp->rc_mode & RLK_LAYER_2_MODE)
				printf("LAYER 2");
			else if (rcp->rc_mode & RLK_LAYER_3_MODE)
				printf("LAYER 3");

			if ((rcp->rc_mode & RLK_BALANCE_MODE) == 0) {
				if (rcp->rc_mode & ~RLK_BALANCE_MODE)
					printf(", ");

				printf("BALANCE_OFF");
			}

			printf(">");
		}
		ppa = rcp->rc_configured_ports[0];
		printf(" = znb%d", ppa);

		rc2.rc_ppa = ppa;
		rc2.rc_dev_id = -1;
		rc2.rc_type = 0;
		rc2.rc_configuration.rc_offset_ports = 0;
		if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc2) != 0) {
			fprintf(stderr, "RMAPI1 protocol error reading configuration for ppa %d\n", ppa);
			return;
		}

		rcp2 = &rc2.rc_configuration;

		if (rcp2->rc_linkup)
			printf("<UP>");
		else
			printf("<DOWN>");

		for (j=1;j<rcp->rc_num_configured_ports;j++) {
			ppa = rcp->rc_configured_ports[j];
			rc2.rc_ppa = ppa;
			rc2.rc_dev_id = -1;
			rc2.rc_type = 0;
			rc2.rc_configuration.rc_offset_ports = 0;
			if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc2) != 0) {
				fprintf(stderr, "RMAPI2 protocol error reading configuration for ppa %d\n", ppa);
				return;
			}
			rcp2 = &rc2.rc_configuration;

			printf(", znb%d", ppa);
			if (rcp2->rc_linkup)
				printf("<UP>");
			else
				printf("<DOWN>");
		}

		/* If not in a fail over group then it is an available device */
		if (rcp->rc_num_primary_ports < 2)
			printf(" ==> zrl%d", dev_id);

		printf("\n");
	}

	/* look for failover groups */
	for (dev_id=0;dev_id<PSDB_MAX_PORTS;dev_id++) {
		rc.rc_ppa = -1;
		rc.rc_dev_id = dev_id;
		rc.rc_type = RMAPI_CONFIG_FAILOVER_TYPE;
		rc.rc_configuration.rc_offset_ports = 0;
		if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc) != 0)
			continue;

		rcp = &rc.rc_configuration;
		no_rainlink = 0;

		if (rcp->rc_version != RC_VERSION) {
			fprintf(stderr, "Version(%d) mismatch with RLK layer version(%d) in driver.  Need updated rlconfig program\n", RC_VERSION, rcp->rc_version);
			return;
		}

		if (rcp->rc_num_primary_ports < 2)
			continue;

		/* Is this the assigned port for the failover group */
		if (rcp->rc_failover_ppa != rcp->rc_ppa)
			continue;


		printf("failover%d", dev_id);
                mode = rcp->rc_mode & (RLK_TIMEOUT_MODE|RLK_HUB_MODE | RLK_SNAP_MODE);
                if (mode != 0) {
                        printf("<");
                        if (mode & RLK_TIMEOUT_MODE)
                                printf("TIMEOUT %d", rcp->rc_rx_timeout);

                        if (mode & RLK_HUB_MODE) {
                                if (mode & RLK_TIMEOUT_MODE)
                                        printf(", ");

                                printf("HUB");
                        }
                        if(mode & RLK_SNAP_MODE){
                                if (mode & ~RLK_SNAP_MODE)
                                        printf(", ");
                                printf("SNAP_BCAST_MODE");
			}

                        printf(">");
                }

                printf(" = ");

		for (j=0;j<rcp->rc_num_primary_ports;j++) {
			if (j != 0)
				printf(",");

			ppa = rcp->rc_primary_ports[j];
			rc2.rc_ppa = ppa;
			rc2.rc_dev_id = -1;
			rc2.rc_type = 0;
			rc2.rc_configuration.rc_offset_ports = 0;
			if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc2) != 0) {
				fprintf(stderr, "RMAPI4 protocol error reading configuration for ppa %d\n", ppa);
				return;
			}
			rcp2 = &rc2.rc_configuration;

			if (rcp2->rc_num_configured_ports > 1) {
				ppa = rmapi_ppa_to_dev(ppa, RMAPI_CONFIG_TRUNK_TYPE);
				printf("trunk%d", ppa);
				if (rcp->rc_failover_port == rcp2->rc_ppa) {
					if (rcp2->rc_num_active_ports == 0)
						printf("<ACTIVE, DOWN>");
					else
						printf("<ACTIVE, UP>");
				}
				else if (rcp2->rc_num_active_ports == 0)
					printf("<DOWN>");
				else
					printf("<STANDBY>");
			}
			else {
				printf("znb%d", ppa);

				if (rcp->rc_failover_port == rcp2->rc_ppa) {
					if (rcp2->rc_linkup)
						printf("<ACTIVE, UP>");
					else
						printf("<ACTIVE, DOWN>");
				}
				else if (rcp2->rc_linkup)
					printf("<STANDBY>");
				else
					printf("<DOWN>");
			}
		}
		printf(" ==> zrl%d\n", dev_id);
	}

	if (no_rainlink)
		printf("No RAINlink features enabled\n");
}
char *hs_state_names[] = {"HS_STATE_NOT_INIT", "HS_STATE_STOPPED", "HS_STATE_RUNNING", "HS_STATE_RELEASED"};

char *hs_fault_names[] = {"HS_FAULT_NONE", "HS_FAULT_INTERNAL", "HS_FAULT_EXTERNAL"};

char *hs_link_state_names[] = {"HS_LINK_AUTO", "HS_LINK_TP", "HS_LINK_BNC", "HS_LINK_AUI", "HS_LINK_TPFD", "HS_LINK_TX", "HS_LINK_TXFD", "HS_LINK_T4"};

void
rc_port_state(ppa)
int32_t ppa;
{
	rmapi_hal_get_state_t rh;
	hal_state_t *hsp;
	uint32_t err;

	rh.rhgs_ppa = ppa;
	err = rmapi_ctl(rmapi_fd, RMAPI_CTL_HAL_GET_STATE, &rh);

	if (err != 0) {
		fprintf(stderr, "Error %d returned while getting state\n", err);
		return;
	}

	hsp = &rh.rhgs_state;
		
	if (hsp->hs_version != HS_VERSION) {
		fprintf(stderr, "Version(%d) mismatch with HAL layer version(%d) in driver.  Need updated rlconfig program\n", HS_VERSION, hsp->hs_version);
		return;
	}

	printf("\t\t\t\t  PORT %2d\n", ppa);
	printf("Structure Version:		%2u\n", hsp->hs_version);
	printf("State:		%18s", hs_state_names[hsp->hs_state]);
	printf("\tFault:	%26s\n", hs_fault_names[hsp->hs_fault]);
	printf("Link configuration: %14s", hs_link_state_names[hsp->hs_link_cfg]);
	if (hsp->hs_link_state == 0)
		printf("\tCurrent link state:   HS_LINK_DOWN\n");
	else
		
		printf("\tCurrent link state: %14s\n", hs_link_state_names[hsp->hs_link_state]);
	printf("Transmit maximum resources:	%2u", hsp->hs_tx_resources_max);
	printf("\tReceive maximum resources:	%2u\n", hsp->hs_rx_resources_max);
	printf("Transmit resources in use:	%2u", hsp->hs_tx_resources);
	printf("\tReceive current resources:	%2u\n", hsp->hs_rx_resources);
	printf("Transmit resources low water:	%2u", hsp->hs_tx_lwm);
	printf("\tReceive resources low water:	%2u\n", hsp->hs_rx_lwm);
	printf("Transmit resources depleted: %5u", hsp->hs_tx_depleted);
	printf("\tReceive resources depleted: %6u\n", hsp->hs_rx_depleted);
	printf("Transmit packets:	%10u", hsp->hs_tx_packets);
	printf("\tReceive packets:	%10u\n", hsp->hs_rx_packets);
	printf("Transmit Descriptors:	%10u", hsp->hs_tx_descriptor_count);
	printf("\tReceive Descriptors:	%10u\n", hsp->hs_rx_descriptor_count);
	printf("Transmit Interrupts	%10u", hsp->hs_tx_icount);
	printf("\tReceive Interrupts	%10u\n", hsp->hs_rx_icount);
	printf("Transmit error packets:	%10u", hsp->hs_tx_err_packets);
	printf("\tReceive error packets:	%10u\n", hsp->hs_rx_err_packets);
	printf("Transmit single collisions:	%2u", hsp->hs_tx_one_collision);
	printf("\tReceive collisions:	%10u\n", hsp->hs_rx_collisions);
	printf("Transmit multiple collisions:	%2u", hsp->hs_tx_more_collisions);
	printf("\tReceive MII error:	%10u\n", hsp->hs_rx_mii_error);
	printf("Transmit maximum collisions:	%2u", hsp->hs_tx_max_collisions);
	printf("\tReceive CRC error:	%10u\n", hsp->hs_rx_crc);
	printf("Transmit late collisions:	%2u", hsp->hs_tx_late_collisions);
	printf("\tReceive descriptor error:	%2u\n", hsp->hs_rx_descriptor_error);
	printf("Transmit carrier loss:		%2u", hsp->hs_tx_carrier_loss);
	printf("\tReceive frame too long:		%2u\n", hsp->hs_rx_frame_too_long);
	printf("Transmit no carrier:		%2u", hsp->hs_tx_no_carrier);
	printf("\tReceive runt frame:	%10u\n", hsp->hs_rx_runt_frame);
	printf("Transmit jabber:		%2u", hsp->hs_tx_jabber);
	printf("\tReceive alignment error:	%2u\n", hsp->hs_rx_alignment_error);
	printf("Transmit underrun:		%2u", hsp->hs_tx_underrun);
	printf("\tReceive overrun:	%10u\n", hsp->hs_rx_overrun);
	printf("Transmit overrun:		%2u", hsp->hs_tx_overrun);
	printf("\tOther Interrupts:	%10u\n", hsp->hs_other_icount);
	printf("Transmit deferred:		%2u\n", hsp->hs_tx_deferred);
}

void
rc_teardown()
{
	rmapi_configuration_t rc;
	rlk_configuration_t *rcp;
	int32_t dev_id;
	uint32_t err;

	/* Teardown failover groups */
	for (dev_id=0;dev_id<PSDB_MAX_PORTS;dev_id++) {
		rmapi_teardown_failover_t rtf;
		
		rc.rc_ppa = -1;
		rc.rc_dev_id = dev_id;
		rc.rc_type = RMAPI_CONFIG_FAILOVER_TYPE;
		rc.rc_configuration.rc_offset_ports = 0;
		if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc) != 0)
			continue;

		rcp = &rc.rc_configuration;

		if ((rcp->rc_num_primary_ports < 2) ||
		    (rcp->rc_failover_ppa != rcp->rc_ppa)) {

			continue;
		}

		rtf.rtf_dev_id = dev_id;
		if ((err = rmapi_ctl(rmapi_fd, RMAPI_CTL_TEARDOWN_FAILOVER, &rtf)) != 0) {
			switch(err) {
				case RMAPI_FAILOVER_IN_USE:
				case RMAPI_TRUNK_IN_USE:
					printf("Bring interface %d down prior to tearing down the trunk\n", dev_id);
					break;
				case RMAPI_LAYER_PUSH_ERROR:
					printf("No RAINlink features enabled\n");
					break;

				default:
					fprintf(stderr, "RMAPI protocol error %d tearing down failover on dev_id %d\n", err, dev_id);
					break;
			}
			return;
		}
	}

	/* Teardown trunks */
	for (dev_id=0;dev_id<PSDB_MAX_PORTS;dev_id++) {
		rmapi_teardown_trunk_t rtt;

		rc.rc_ppa = -1;
		rc.rc_dev_id = dev_id;
		rc.rc_type = RMAPI_CONFIG_TRUNK_TYPE;
		rc.rc_configuration.rc_offset_ports = 0;
		if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc) != 0)
			continue;

		rcp = &rc.rc_configuration;

		if (rcp->rc_num_configured_ports < 2)
			continue;

		rtt.rtt_dev_id = dev_id;
		if ((err = rmapi_ctl(rmapi_fd, RMAPI_CTL_TEARDOWN_TRUNK, &rtt)) != 0) {
			switch(err) {
				case RMAPI_FAILOVER_IN_USE:
				case RMAPI_TRUNK_IN_USE:
					printf("Bring interface %d down prior to tearing down the trunk\n", dev_id);
					break;
				case RMAPI_LAYER_PUSH_ERROR:
					printf("No RAINlink features enabled\n");
					break;

				default:
					fprintf(stderr, "RMAPI protocol error %d tearing down trunk on dev_id %d\n", err, dev_id);
					break;
			}
			return;
		}
	}

	/* Check teardown */
	for (dev_id=0;dev_id<PSDB_MAX_PORTS;dev_id++) {
		rc.rc_ppa = -1;
		rc.rc_dev_id = dev_id;
		rc.rc_type = RMAPI_CONFIG_TRUNK_TYPE;
		rc.rc_configuration.rc_offset_ports = 0;
		if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc) != 0)
			continue;

		rcp = &rc.rc_configuration;

		if (rcp->rc_num_primary_ports > 1) {
			fprintf(stderr, "rc_teardown: Failover teardown failed - reload the driver\n");
			return;
		}

		if (rcp->rc_num_configured_ports > 1) {
			fprintf(stderr, "rc_teardown: Trunk teardown failed - reload the driver\n");
			return;
		}
	}
}

rc_create_trunk(dev_id, pa)
int dev_id;
ppa_array_t *pa;
{
	int i;

	if (rc_find_group(pa, dev_id, 0) != 0) {
		fprintf(stderr, "line %d: ", lineno);
		fprintf(stderr, "trunk/failover %d in use\n", dev_id);
		exit(0);
	}
	if (dev_id >= PSDB_MAX_PORTS) {
		fprintf(stderr, "line %d: ", lineno);
		fprintf(stderr, "trunk identifier value %d too large\n", dev_id);
		exit(0);
	}

	pa->pa_is_trunk = 1;
	pa->pa_is_failover = 0;
	pa->pa_dev_id = dev_id;
}

rc_create_failover(dev_id, pa)
int dev_id;
ppa_array_t *pa;
{
	ppa_array_t *trunk_pa;
	int i;

	if ((trunk_pa = rc_find_group(pa, dev_id, 0)) != 0) {
		/* If the trunk_pa is part of this failover group then it is
		 * o.k. to reuse the dev_id.
		 */
		for (i=0;i<pa->pa_index;i++) {
			if (pa->pa_array[i] == trunk_pa->pa_array[0])
				break;
		}

		if (i == pa->pa_index) {
			fprintf(stderr, "line %d: ", lineno);
			fprintf(stderr, "trunk/failover %d in use\n", dev_id);
			exit(0);
		}
	}
	if (dev_id >= PSDB_MAX_PORTS) {
		fprintf(stderr, "line %d: ", lineno);
		fprintf(stderr, "failover identifier value %d too large\n", dev_id);
		exit(0);
	}

	pa->pa_dev_id = dev_id;
	pa->pa_is_trunk = 0;
	pa->pa_is_failover = 1;
}

rc_teardown_trunk(dev_id)
int32_t dev_id;
{
	rmapi_configuration_t rc;
	rlk_configuration_t *rcp;
	rmapi_teardown_trunk_t rtt;
	uint32_t err;

	rc.rc_ppa = -1;
	rc.rc_dev_id = dev_id;
	rc.rc_type = 0;
	rc.rc_configuration.rc_offset_ports = 0;
	if ((rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc)) != 0) {
		fprintf(stderr, "line %d: ", lineno);
		fprintf(stderr, "zrl%d does not exist\n", dev_id);
		exit(0);
	}

	rcp = &rc.rc_configuration;
	if ((rcp->rc_num_primary_ports > 1) ||
		(rcp->rc_num_configured_ports < 2)) {
		fprintf(stderr, "line %d: ", lineno);
		fprintf(stderr, "zrl%d is not a trunk\n",dev_id);
		exit(0);
	}

	rtt.rtt_dev_id = dev_id;
	if ((err = rmapi_ctl(rmapi_fd, RMAPI_CTL_TEARDOWN_TRUNK, &rtt)) != 0) {
		fprintf(stderr, "line %d: ", lineno);
		switch(err) {
			case RMAPI_FAILOVER_IN_USE:
			case RMAPI_TRUNK_IN_USE:
				fprintf(stderr, "Bring interface zrl%d down prior to tearing down the trunk\n", dev_id);
				break;
			case RMAPI_LAYER_PUSH_ERROR:
				fprintf(stderr, "No RAINlink features enabled\n");
				break;

			default:
				fprintf(stderr, "RMAPI protocol error %d tearing down trunk on dev_id %d\n", err, dev_id);
				break;
		}
		exit(0);
	}
	return(0);
}

rc_teardown_failover(dev_id)
int32_t dev_id;
{
	rmapi_configuration_t rc;
	rlk_configuration_t *rcp;
	rmapi_teardown_failover_t rtf;
	uint32_t err;

	/* Teardown failover groups */
	rc.rc_ppa = -1;
	rc.rc_dev_id = dev_id;
	rc.rc_type = 0;
	rc.rc_configuration.rc_offset_ports = 0;
	if ((rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc)) != 0) {
		fprintf(stderr, "line %d: ", lineno);
		fprintf(stderr, "zrl%d does not exist\n", dev_id);
		exit(0);
	}

	rcp = &rc.rc_configuration;
	if ((rcp->rc_num_primary_ports < 2) ||
	    (rcp->rc_failover_ppa != rcp->rc_ppa)) {
		fprintf(stderr, "line %d: ", lineno);
		fprintf(stderr, "zrl%d is not a failover group\n",dev_id);
		exit(0);
	}

	rtf.rtf_dev_id = dev_id;
	if ((err = rmapi_ctl(rmapi_fd, RMAPI_CTL_TEARDOWN_FAILOVER, &rtf)) != 0) {
		fprintf(stderr, "line %d: ", lineno);
		switch(err) {
			case RMAPI_FAILOVER_IN_USE:
			case RMAPI_TRUNK_IN_USE:
				fprintf(stderr, "Bring interface zrl%d down prior to tearing down the trunk\n", dev_id);
				break;
			case RMAPI_LAYER_PUSH_ERROR:
				fprintf(stderr, "No RAINlink features enabled\n");
				break;

			default:
				fprintf(stderr, "RMAPI protocol error %d tearing down failover on dev_id %d\n", err, dev_id);
				break;
		}
		exit(0);
	}
	return(0);
}

#define PA_STRING(pa) (pa->pa_is_trunk ? "TRUNK" : (pa->pa_is_failover ? "FAILOVER" : "ZNB"))

rc_process()
{
	ppa_array_t *pa;
	uint32_t ppa;
	uint32_t dev_id;
	int i;

	pa = 0;
	while ((pa = (ppa_array_t *) pdk_q_next((pdk_q_t **) &ppa_arrays, (pdk_q_t *) pa)) != 0) {
		DEBUG2("rc_process(%d): %s 0x%x with mode 0x%x and clear_mode 0x%x\n", pa->pa_dev_id, PA_STRING(pa), pa, pa->pa_mode, pa->pa_clear_mode);
		if (pa->pa_is_trunk) {
			if (pa->pa_is_built == 0) {
				rmapi_create_trunk_t rct;

				rct.rct_dev_id = pa->pa_dev_id;
				rct.rct_num_ppa = pa->pa_index;
				for (i=0;i<pa->pa_index;i++)
					rct.rct_ppa_array[i] = pa->pa_array[i];

				if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CREATE_TRUNK, &rct) != 0) {
					printf("TRUNK%d Not created\n", pa->pa_dev_id);
					pa->pa_is_trunk = 0;
					continue;
				}

				printf("TRUNK%d Successfully Created\n", pa->pa_dev_id);
			}
			if (pa->pa_mode != 0) {
				rmapi_set_mode_t rsm;

				rsm.rsm_dev_id = pa->pa_dev_id;
				rsm.rsm_mode = pa->pa_mode;
				rsm.rsm_time = pa->pa_rx_timeout;
				if (rmapi_ctl(rmapi_fd, RMAPI_CTL_SET_MODE, &rsm) == 0)
					printf("TRUNK%d mode set\n", pa->pa_dev_id);
				else
					printf("TRUNK%d mode set failed\n", pa->pa_dev_id);
			}
			else if (pa->pa_clear_mode != 0) {
				rmapi_clear_mode_t rcm;

				rcm.rcm_dev_id = pa->pa_dev_id;
				rcm.rcm_mode = pa->pa_clear_mode;
				if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CLEAR_MODE, &rcm) == 0)
					printf("TRUNK%d mode cleared\n", pa->pa_dev_id);
				else
					printf("TRUNK%d mode clear failed\n", pa->pa_dev_id);
			}
		}
		else if (pa->pa_is_failover) {
			if (pa->pa_is_built == 0) {
				rmapi_create_failover_t rcf;

				rcf.rcf_dev_id = pa->pa_dev_id;
				rcf.rcf_num_ppa = pa->pa_index;
				for (i=0;i<pa->pa_index;i++)
					rcf.rcf_ppa_array[i] = pa->pa_array[i];

				if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CREATE_FAILOVER, &rcf) != 0) {
					printf("FAILOVER%d Not created\n", pa->pa_dev_id);
					pa->pa_is_failover = 0;
					continue;
				}
				printf("FAILOVER%d Successfully Created\n", pa->pa_dev_id);
			}
			if (pa->pa_mode != 0) {
				rmapi_set_mode_t rsm;

				rsm.rsm_dev_id = pa->pa_dev_id;
				rsm.rsm_mode = pa->pa_mode;
				rsm.rsm_time = pa->pa_rx_timeout;
				if (rmapi_ctl(rmapi_fd, RMAPI_CTL_SET_MODE, &rsm) == 0)
					printf("FAILOVER%d mode set\n", pa->pa_dev_id);
				else
					printf("FAILOVER%d mode set failed\n", pa->pa_dev_id);
			}
			else if (pa->pa_clear_mode != 0) {
				rmapi_clear_mode_t rcm;

				rcm.rcm_dev_id = pa->pa_dev_id;
				rcm.rcm_mode = pa->pa_clear_mode;
				if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CLEAR_MODE, &rcm) == 0)
					printf("FAILOVER%d mode cleared\n", pa->pa_dev_id);
				else
					printf("FAILOVER%d mode clear failed\n", pa->pa_dev_id);
			}
		}
	}
}

void
rc_list(format)
int format;
{
	rmapi_configuration_t rc;
	rlk_configuration_t *rcp;
	int not_znb[PSDB_MAX_PORTS];
	int first_print;
	int32_t ppa, dev_id;
	int i, j;

	if (format != 0) {
		usage("Only list format supported is 0");
		exit(1);
	}

	for (i=0;i<PSDB_MAX_PORTS;i++)
		not_znb[i] = 0;

	first_print = 1;
	fprintf(flist, "ZRL=\"");

	for (ppa=0;ppa<PSDB_MAX_PORTS;ppa++) {
		rc.rc_ppa = ppa;
		rc.rc_dev_id = -1;
		rc.rc_type = 0;
		rc.rc_configuration.rc_offset_ports = 0;
		if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc) != 0)
			continue;

		rcp = &rc.rc_configuration;

		if (rcp->rc_version != RC_VERSION) {
			fprintf(stderr, "Version(%d) mismatch with RLK layer version(%d) in driver.  Need updated rlconfig program\n", RC_VERSION, rcp->rc_version);
			return;
		}

		if ((rcp->rc_num_primary_ports < 2) && (rcp->rc_num_configured_ports < 2))
			continue;

		for (j=0;j<rcp->rc_num_configured_ports;j++)
			not_znb[rcp->rc_configured_ports[j]] = 1;

		if (rcp->rc_num_primary_ports > 1) {
			/* Must be the assigned port for
			 * the failover group */
			if (rcp->rc_failover_ppa != rcp->rc_ppa)
				continue;

			dev_id = rmapi_ppa_to_dev(rcp->rc_ppa, RMAPI_CONFIG_FAILOVER_TYPE);
		}
		else {
			dev_id = rmapi_ppa_to_dev(rcp->rc_ppa, RMAPI_CONFIG_TRUNK_TYPE );
		}

		if (first_print) {
			first_print = 0;
			fprintf(flist, "%d", dev_id);
		}
		else
			fprintf(flist, " %d", dev_id);
	}
	fprintf(flist, "\"\n");

	first_print = 1;
	fprintf(flist, "ZNB=\"");
	for (i=0;i<PSDB_MAX_PORTS;i++) {
		if (not_znb[i] == 0) {
			rc.rc_ppa = i;
			rc.rc_dev_id = -1;
			rc.rc_type = 0;
			rc.rc_configuration.rc_offset_ports = 0;
			if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc) == RMAPI_BAD_DEVICE_ID)
				continue;

			if (first_print) {
				first_print = 0;
				fprintf(flist, "%d", i);
			}
			else
				fprintf(flist, " %d", i);
		}
	}
	fprintf(flist, "\"\n");
}

usage(s)
char *s;
{
	printf("%s\n", s);
	printf("Usage: rlconfig [-s][-p <ppa>][-t][-l[#] <list file>] -h <host_name> < <Input File>\n");
	printf("                -s: show current configuration\n");
	printf("                -p: show hardware level state of specified <ppa>\n");
	printf("                -t: teardown configuration\n");
	printf("                -l: create <list file> useful as input to\n");
	printf("                    configuration scripts.  Optional number\n");
	printf("                    specifies format.  Default is 0.\n");
	printf("                -h: connect to remote host <host_name>\n");
}

yyerror(const char *msg)
{
	(void) fprintf(stderr, "line %d: %s\n", lineno, msg);
	exit(0);
}


