/*==========================================================================
**
**	./pdk/src/drivers/apps/rlutils/rld.c
**
**	RAINlink redirector deamon
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	ram	11/4/99 v1.0	Created
**
**==========================================================================
**
**	Notes:
**==========================================================================
*/

#include <stdio.h>
#include <pdk_port.h>
#undef _KERNEL
#include <pdk_global.h>
#include <pdk_types.h>

#include <pdk_errno.h>
#include <errno.h>

#include <pdk_queue.h>

#include <rmapi.h>
#define _REENTRANT

#include <pthread.h>
#ifdef pthread_attr_create
#define RPA rpa
#else
#define RPA NULL
#endif

#include <signal.h>
#include <socket.h>
#include <netinet/in.h>

#include <psdb_io.h>
#include <stdarg.h>

#include <time.h>

int32_t rld_debug = 1;
int32_t rld_logging = 0;
int32_t rmapi_debug;

FILE *rld_log;

void rld_print(int32_t level, char *s, va_list argptr);
void rld_print0(char *s, ...);
void rld_print1(char *s, ...);
void rld_print2(char *s, ...);
void rld_print3(char *s, ...);
void rld_print4(char *s, ...);

#define DEBUG0 rld_print0
#define DEBUG1 if ((rld_debug>0) || (rld_logging>0)) rld_print1
#define DEBUG2 if ((rld_debug>1) || (rld_logging>1)) rld_print2
#define DEBUG3 if ((rld_debug>2) || (rld_logging>2)) rld_print3
#define DEBUG4 if ((rld_debug>3) || (rld_logging>3)) rld_print4

#define RLD_LISTEN_PORT 7000

#define RLD_LOG_FILE "/etc/rain/rld.log"

void *rld_handler(void *vp);
void rld_send_rmapi_error(int sock_fd, int32_t type, int32_t err);
void rld_send_rmapi_success(int sock_fd, int32_t type);
void rld_send_rmapi(int sock_fd, PSDB_RHDR *pprp);
void rld_io_get(int sock_fd, rmapi_fd_t fd, PSDB_RHDR *pprp, int32_t len);
#ifdef POLL
void *rld_poll(void *time);
#endif
void rld_closedown(int sig);
void rld_status(int sig);

#define RLD_MAX(a, b) ((a) > (b) ? (a) : (b))

typedef struct rld_process {
	pthread_t	rp_tid;
	int32_t		rp_is_active;
	int		rp_fd;
} rld_process_t;

#define NUM_RLD_PROCESS 32
int32_t rld_process_index = 0;
rld_process_t rld_process[NUM_RLD_PROCESS];

#define RLD_POP_PROCESS(rp) { \
	int32_t to, from; \
\
	for (from=0, to=0;from<rld_process_index;from++) \
		if (rp != &rld_process[from]) \
			rld_process[to++] = rld_process[from]; \
		else \
			rp->rp_is_active = 0; \
 \
	rld_process_index = to; \
}

pthread_mutex_t m;

extern char *optarg;
extern int errno;


main(argc, argv)
int	argc;
char	*argv[];
{
	pthread_t tid;
	int32_t rld_listen_port;
	int32_t foreground;
	char c;

	foreground = 0;
	rld_listen_port = RLD_LISTEN_PORT;

	while ((c = getopt(argc, argv, "d:p:fl:")) != -1) {
		switch (c) {
		case 'd':
			rld_debug = atoi(optarg);
			rmapi_debug = rld_debug;
			break;

		case 'l':
			rld_logging = atoi(optarg);
			rld_log = fopen(RLD_LOG_FILE, "w");
			break;

		case 'p':
			rld_listen_port = atoi(optarg);
			break;

		case 'f':
			foreground++;
			break;

		default:
			usage("");
			return(-1);
		}
	}

	if (foreground == 0) {
		if (fork() != 0) {
			exit(0);
		}
	}

	pthread_mutex_init(&m, NULL);

#ifdef POLL
	{
#ifdef pthread_attr_create
		pthread_attr_t rpa;

		pthread_attr_create(&rpa);
#endif
		if (pthread_create(&tid, RPA , rld_poll, (void *) 3) != 0) {
			perror("pthread_create");
			DEBUG1("rld_listen:  pthread_create failed\n");
			exit(0);
		}
#ifdef LYNXOS
		pthread_detach(&tid);
#else
		pthread_detach(tid);
#endif /* LYNXOS */
	}
#endif

/*
	close(0);
	close(1);
*/
	rld_listen(rld_listen_port);
}

int parent_sock;

rld_listen(port)
int32_t port;
{
	rld_process_t *rp;
	struct sockaddr_in sin;
	int tmp = 1;
#ifdef pthread_attr_create
	pthread_attr_t rpa;

	pthread_attr_create(&rpa);
#endif

	parent_sock = socket(AF_INET, SOCK_STREAM, 0);

	(void) signal(SIGHUP, rld_closedown);
	(void) signal(SIGTERM, rld_closedown);
	(void) signal(SIGINT, rld_closedown);
	(void) signal(SIGQUIT, rld_closedown);
	(void) signal(SIGUSR1, rld_status);


	if (parent_sock < 0) {
		DEBUG1("rld_listen:  Could not create TCP socket on port %d\n", port);
		exit(0);
	}

	setsockopt(parent_sock, SOL_SOCKET, SO_REUSEADDR, (const void *) tmp, sizeof(tmp));

	memset(&sin, 0, sizeof(struct sockaddr_in));

	sin.sin_family = AF_INET;
	sin.sin_port = htons(port);
	sin.sin_addr.s_addr = htonl(INADDR_ANY);
	if (bind(parent_sock, (struct sockaddr *) &sin, sizeof(struct sockaddr_in)) != 0) {
		perror("bind");
		close(parent_sock);
		DEBUG1("rld_listen:  Could not bind on TCP socket on port %d\n", port);
		exit(0);
	}


	if (listen(parent_sock, 5) != 0) {
		perror("listen");
		close(parent_sock);
		DEBUG1("rld_listen:  Could not listen on TCP socket on port %d\n", port);
		exit(0);
	}
	

	while (1) {
		int sock_len;
		int sock_fd;

		memset(&sin, 0, sizeof(struct sockaddr_in));
		sock_len = sizeof(struct sockaddr_in);

		sock_fd = accept(parent_sock, (struct sockaddr *) &sin, &sock_len);
		DEBUG2("rld_listen:  Accepted connection from 0x%x\n", sin.sin_addr.s_addr);

		if (sock_fd < 0) {
			if (errno == EINTR)
				continue;

			perror("accept");
			close(parent_sock);
			DEBUG1("rld_listen:  Accept failed\n");
			exit(0);
		}
		setsockopt(sock_fd, SOL_SOCKET, SO_KEEPALIVE, (const void *) tmp, sizeof(tmp));

		pthread_mutex_lock(&m);
		rp = &rld_process[rld_process_index++];
		rp->rp_is_active = 0;
		rp->rp_fd = sock_fd;
		pthread_mutex_unlock(&m);
		if (pthread_create(&rp->rp_tid, RPA, rld_handler, (void *) rp) != 0) {
			pthread_mutex_lock(&m);
			RLD_POP_PROCESS(rp);
			pthread_mutex_unlock(&m);
			perror("pthread_create");
			DEBUG1("rld_listen:  pthread_create failed\n");
			close(sock_fd);
			if (errno != EAGAIN) {
				close(parent_sock);
				exit(0);
			}
		}

#ifdef LYNXOS
		pthread_detach(&rp->rp_tid);
#else
		pthread_detach(rp->rp_tid);
#endif /* LYNXOS */
	}
}

/* Maximum value of read */
#define RMAPI_BUFSIZE 4096

void *
rld_handler(vp)
void *vp;
{
	rld_process_t *rp;
	int sock_fd;
	char buf[RMAPI_BUFSIZE];
	PSDB_RHDR *pprp;
	uint32_t *lp, *record_lim;
	int32_t record_size, size, type;
	int32_t err;
	int32_t read_flag;
	rmapi_fd_t fd;
	int32_t first_loop;

	DEBUG2("rld_handler(0x%x):  Launching\n", vp);

	pthread_mutex_lock(&m);
	rp = (rld_process_t *) vp;
	rp->rp_is_active = 1;
	sock_fd = rp->rp_fd;
	pthread_mutex_unlock(&m);

	fd = psdb_io_open(0, 0);

	pprp = (PSDB_RHDR *) buf;
	first_loop = 1;
	while (recv(sock_fd, (void *) pprp, sizeof(PSDB_RHDR), 0) > 0) {

		record_size = psdb_pton(pprp->Size);
		type = psdb_pton(pprp->Type);

		DEBUG3("rld_handler(0x%x): Record size %d, type 0x%x\n", vp, record_size, type);

		if (record_size > RMAPI_BUFSIZE) {
			DEBUG1("rld_handler(0x%x): RMAPI protocol error - packet too large %d bytes\n", vp, record_size);
			rld_send_rmapi_error(sock_fd, type, RMAPI_PROTOCOL_ERROR);
			continue;
		}

		size = sizeof(PSDB_RHDR);

		while (size < record_size) {
			int read_size;

			read_size = read(sock_fd, buf + size, record_size-size);
			if (read_size == EOF)
				break;

			size += read_size;
		}

		if (size != record_size) {
			DEBUG1("rld_handler(0x%x):  RMAPI protocol error - read incorrect number of bytes\n", vp);
			rld_send_rmapi_error(sock_fd, type, RMAPI_PROTOCOL_ERROR);
			continue;
		}

		switch(type) {
			case PSDB_TYPE_IO_OPEN:
				if (first_loop == 0) {
					rld_send_rmapi_error(sock_fd, type, RMAPI_PROTOCOL_ERROR);
					DEBUG1("rld_handler(0x%x):  Recieved PSDB_IO_OPEN in middle of protocol stream\n", vp);
					pthread_mutex_lock(&m);
					RLD_POP_PROCESS(rp);
					pthread_mutex_unlock(&m);
					psdb_io_close(fd);
					close(sock_fd);
					return(0);
				}

				rld_send_rmapi_success(sock_fd, type);
				/* Ignore */
				first_loop = 0;
				continue;

			case PSDB_TYPE_IO_CLOSE:
				rld_send_rmapi_success(sock_fd, type);
				DEBUG2("rld_handler(0x%x):  psdb_io_close IO_CLOSE called\n", vp);

				pthread_mutex_lock(&m);
				RLD_POP_PROCESS(rp);
				pthread_mutex_unlock(&m);

				psdb_io_close(fd);

				close(sock_fd);
				pthread_exit(0);

			case PSDB_TYPE_IO_GET: {
				pdk_psdb_io_get_t *piogp;

				piogp = (pdk_psdb_io_get_t *) pprp;

				DEBUG3("rld_handler(0x%x):  PSDB_IO_GET: len %d\n", vp, psdb_pton(piogp->piog_len));
				rld_io_get(sock_fd, fd, pprp, psdb_pton(piogp->piog_len));
				continue;
			}
		}

		read_flag = PSDB_READ_FLAG(type);

		first_loop = 0;

		if ((err = psdb_io_send(fd, (pdk_psdb_rhdr_t *) pprp, read_flag)) != 0) {
			rld_send_rmapi_error(sock_fd, type, err);
			continue;
		}

		rld_send_rmapi_success(sock_fd, type);

		if (read_flag)
			rld_send_rmapi(sock_fd, pprp);

	}

	pthread_mutex_lock(&m);
	RLD_POP_PROCESS(rp);
	pthread_mutex_unlock(&m);

	DEBUG2("rld_handler(0x%x):  psdb_io_close called\n", vp);
	psdb_io_close(fd);

	close(sock_fd);
	return(0);
}

void
rld_io_get(sock_fd, fd, pprp, len)
int sock_fd;
rmapi_fd_t fd;
PSDB_RHDR *pprp;
int32_t len;
{
	fd_set rfs, efs;
	int32_t err;

	FD_ZERO(&rfs);
	FD_ZERO(&efs);
	FD_SET(sock_fd, &rfs);
	FD_SET(psdb_io_fd(fd), &rfs);

	FD_SET(sock_fd, &efs);
	FD_SET(psdb_io_fd(fd), &efs);

	select(RLD_MAX(sock_fd, psdb_io_fd(fd)) + 1, &rfs, (fd_set *) 0, &efs, (struct timeval *) 0);

	if (FD_ISSET(sock_fd, &rfs) || FD_ISSET(sock_fd, &efs) || FD_ISSET(psdb_io_fd(fd), &efs)) {
		/* Remote side must have broken out of the io_get */
		DEBUG2("rld_io_get:  Broken pipe on socket revents\n");
		return;
	}

	if (FD_ISSET(psdb_io_fd(fd), &rfs) == 0) {
		/* Error on the stream. */
		DEBUG2("rld_io_get:  Error on file io\n");
		rld_send_rmapi_error(sock_fd, PSDB_TYPE_IO_GET, RMAPI_PROTOCOL_ERROR);
		return;
	}

	if ((err = psdb_io_get(fd, (pdk_psdb_rhdr_t *) pprp, len)) != 0)
		rld_send_rmapi_error(sock_fd, PSDB_TYPE_IO_GET, err);
	else {
		rld_send_rmapi_success(sock_fd, PSDB_TYPE_IO_GET);
		rld_send_rmapi(sock_fd, pprp);
	}
}

void
rld_send_rmapi_error(sock_fd, type, err)
int sock_fd;
int32_t err;
{
	pdk_psdb_error_t pe;

	pe.pe_header.ppr_header.Size = psdb_ntop(sizeof(pdk_psdb_error_t));
	pe.pe_header.ppr_header.Type = psdb_ntop(PSDB_TYPE_ERROR);

	pe.pe_error_type = psdb_ntop(type);
	pe.pe_error = psdb_ntop(err);

	rld_send_rmapi(sock_fd, (PSDB_RHDR *) &pe);
}

void
rld_send_rmapi_success(sock_fd, type)
int sock_fd;
{
	pdk_psdb_success_t ps;

	ps.ps_header.ppr_header.Size = psdb_ntop(sizeof(pdk_psdb_error_t));
	ps.ps_header.ppr_header.Type = psdb_ntop(PSDB_TYPE_SUCCESS);

	ps.ps_success_type = psdb_ntop(type);

	rld_send_rmapi(sock_fd, (PSDB_RHDR *) &ps);
}

void
rld_send_rmapi(sock_fd, pprp)
int sock_fd;
PSDB_RHDR *pprp;
{
	int32_t size;


	size = psdb_pton(pprp->Size);

	DEBUG3("rld_send_rmapi: type 0x%x size %d\n", psdb_ntop(pprp->Type), size);

	write(sock_fd, pprp, size);
}

#ifdef POLL
#define POLL_INTERVAL 3

void *
rld_poll(time)
void *time;
{
	struct pollfd pfds[NUM_RLD_PROCESS];
	rld_process_t *rpp[NUM_RLD_PROCESS];
	struct timeval tv;
	int i, si;
	fd_set efs;
	int max_fd;

	tv.tv_sec = POLL_INTERVAL;
	tv.tv_usec = 0;

	while (1) {
		pthread_mutex_lock(&m);
		max_fd = 0;
		for (i=0, si=0; i<rld_process_index; i++) {
			if (rld_process[i].rp_is_active) {
				FD_SET(rld_process[i].rp_fd, &efs);

				if (rld_process[i].rp_fd > max_fd)
					max_fd = rld_process[i].rp_fd;

				rpp[si] = &rld_process[i];
				si++;
			}
		}
		pthread_mutex_unlock(&m);
		if (si == 0) {
			DEBUG3("rld_poll:  No active process\n");
			sleep(POLL_INTERVAL);
			continue;
		}

		DEBUG2("rld_poll:  polling %d connections\n", si);
		select(max_fd+1, (fd_set) 0, (fd_set) 0, &efs, (struct timeout *) &tv);
		DEBUG2("rld_poll:  poll complete\n");

		pthread_mutex_lock(&m);

		for (i=0; i<si; i++) {
			DEBUG2("rld_poll: Checking process 0x%x - re %x fd %d %s\n", rpp[i], pfds[i].revents, pfds[i].fd, rpp[i]->rp_is_active ? "ACTIVE" : "INACTIVE");
			if ((pfds[i].revents & (POLLERR|POLLHUP)) &&
			    (pfds[i].fd == rpp[i]->rp_fd) &&
			    (rpp[i]->rp_is_active)) {

				DEBUG1("rld_poll:  Process %d hung\n", i);
				pthread_cancel(rpp[i]->rp_tid);
				RLD_POP_PROCESS(rpp[i]);
			}
			else if (pfds[i].revents & (POLLERR|POLLHUP)) {
				DEBUG2("rld_poll:  Poll noticed proper close down on process 0x%x\n", rpp[si]); 
			}
		}

		pthread_mutex_unlock(&m);
	}
}
#endif /* POLL */

void
rld_status(sig)
int sig;
{
	int32_t i;
	rld_process_t *rp;

	DEBUG0("RLD_PROCESS_STATUS: tid     active      fd\n");
	for (i=0; i<rld_process_index; i++) {
		rp = &rld_process[i];
		DEBUG0("                  0x%8x  %d  %d\n", rp->rp_tid, rp->rp_is_active, rp->rp_fd);
	}

	(void) signal(SIGUSR1, rld_status);
}

void
rld_closedown(sig)
int sig;
{
	int32_t i;
	rld_process_t *rp;
	DEBUG2("rld_closedown(%d):\n", sig);
	close(parent_sock);

	for (i=0; i<rld_process_index; i++) {
		rp = &rld_process[i];
		pthread_cancel(rp->rp_tid);
		rp->rp_is_active = 0;
		close(rp->rp_fd);
	}

	exit(1);
}
	


usage(s)
char *s;
{
	printf("%s\n", s);
	printf("Usage: rld [-d <level>] [-p <port>] [-f] [-l <level>]\n");
	printf("                -d: set the debug level to <level>\n");
	printf("                -p: set listening port.  Devault 7000\n");
	printf("                -f: run deamon in forground\n");
	printf("                -l: log debug messages at <level> to /etc/rain/rld.log\n");
}

void
rld_print0(char *s, ...)
{
	va_list argptr;

	va_start(argptr, s);

	rld_print(0, s, argptr);

	va_end(argptr);
}

void
rld_print1(char *s, ...)
{
	va_list argptr;

	va_start(argptr, s);

	rld_print(1, s, argptr);

	va_end(argptr);
}

void
rld_print2(char *s, ...)
{
	va_list argptr;

	va_start(argptr, s);

	rld_print(2, s, argptr);

	va_end(argptr);
}

void
rld_print3(char *s, ...)
{
	va_list argptr;

	va_start(argptr, s);

	rld_print(3, s, argptr);

	va_end(argptr);
}

void
rld_print4(char *s, ...)
{
	va_list argptr;

	va_start(argptr, s);

	rld_print(4, s, argptr);

	va_end(argptr);
}

void
rld_print(int32_t level, char *s, va_list argptr)
{
	time_t t;
	char *s1;

	if (rld_debug >= level)
		vfprintf(stderr, s, argptr);

	if (rld_logging >= level) {
		time(&t);
		s1 = ctime(&t);
		s1[19] = ':';
		s1[20] = ' ';
		s1[21] = 0;
		fprintf(rld_log, "%s", s1);
		vfprintf(rld_log, s, argptr);
		fflush(rld_log);
	}
}

