/*==========================================================================
**
**	rmapi.c
**
**	RAIN Management API
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	ram	01/12/99 v1.0	Created
**	cbp	10/12/99 v2.0	RMAPI version
**
**==========================================================================
**
**	Notes:
**==========================================================================
*/


#include <pdk_port.h>
#undef _KERNEL
#include <pdk_global.h>
#include <pdk_types.h>

#include <pdk_psdb.h>

#include <psdb_io.h>
#include <rmapi.h>

int32_t rmapi_debug = 0;

#define DEBUG1 if (rmapi_debug>0) printf
#define DEBUG2 if (rmapi_debug>1) printf
#define DEBUG3 if (rmapi_debug>2) printf
#define DEBUG4 if (rmapi_debug>3) printf

pdk_psdb_rhdr_t *rmapi_to_psdb(uint32_t opcode, int32_t size, void *param);

rmapi_fd_t
rmapi_open(dev, flags)
char *dev;
uint32_t flags;
{
	return(psdb_io_open(dev, flags));
}

uint32_t
rmapi_close(fd)
rmapi_fd_t fd;
{
	return(psdb_io_close(fd));
}

uint32_t
rmapi_ctl(fd, opcode, param)
rmapi_fd_t fd;
uint32_t opcode;
void *param;
{
	pdk_psdb_rhdr_t *prp;
	int32_t read_flag;
	uint32_t *lp, *dp, *record_lim;
	int32_t size;
	uint32_t err;

	switch (opcode) {
		case RMAPI_CTL_CONFIGURATION: {
			rmapi_configuration_t *rcp = (rmapi_configuration_t *)param;
			size = sizeof(rmapi_configuration_t);
			rcp->rc_configuration.rc_version = RC_VERSION;
			rcp->rc_configuration.rc_num_ports = PSDB_MAX_PORTS;
			break;
		}

		case RMAPI_CTL_CREATE_TRUNK: {
			rmapi_create_trunk_t *rctp = (rmapi_create_trunk_t *)param;
			size = sizeof(pdk_psdb_create_trunk_t);
			size += sizeof(int32_t)*(rctp->rct_num_ppa-1);
			break;
		}

		case RMAPI_CTL_CREATE_FAILOVER: { 
			rmapi_create_failover_t *rcfp = (rmapi_create_failover_t *)param;
			size = sizeof(pdk_psdb_create_failover_t);
			size += sizeof(int32_t)*(rcfp->rcf_num_ppa-1);
			break;
		}
		case RMAPI_CTL_TEARDOWN_TRUNK:
			size = sizeof(rmapi_teardown_trunk_t);
			break;

		case RMAPI_CTL_TEARDOWN_FAILOVER:
			size = sizeof(rmapi_teardown_failover_t);
			break;

		case RMAPI_CTL_SET_MODE:
			size = sizeof(rmapi_set_mode_t);
			break;

		case RMAPI_CTL_CLEAR_MODE:
			size = sizeof(rmapi_clear_mode_t);
			break;

		case RMAPI_CTL_TEST:
			size = sizeof(rmapi_test_t);
			break;

		case RMAPI_CTL_FAILOVER:
			size = sizeof(rmapi_failover_t);
			break;

		case RMAPI_CTL_HAL_GET_STATE:
			size = sizeof(rmapi_hal_get_state_t);
			break;

		case RMAPI_CTL_HAL_SPEEDOMETER_GET:
			size = sizeof(rmapi_hal_speedometer_get_t);
			break;

		case RMAPI_CTL_HAL_SPEEDOMETER_SET_PARAMS:
			size = sizeof(rmapi_hal_speedometer_set_params_t);
			break;

		case RMAPI_CTL_HAL_FAULT_NOTIFICATION_SET:
		case RMAPI_CTL_HAL_FAULT_NOTIFICATION_CLEAR:
			size = sizeof(rmapi_hal_fault_t);
			break;

		case RMAPI_CTL_PAL_SET_MODE:
			size = sizeof(rmapi_pal_set_mode_t);
			break;

		case RMAPI_CTL_PAL_CLEAR_MODE:
			size = sizeof(rmapi_pal_clear_mode_t);
			break;

		case RMAPI_CTL_PAL_GET_HW_INFO:
			size = sizeof(rmapi_pal_get_hw_info_t);
			break;

		default:
			return(RMAPI_PROTOCOL_ERROR);
	}

	read_flag = RMAPI_READ_FLAG(opcode);

	prp = rmapi_to_psdb(opcode, size, param);

	if ((err = psdb_io_send(fd, prp, read_flag)) != 0)
		return(err);

	if (read_flag) {
		lp = (uint32_t *) (prp+1);
		record_lim = (uint32_t *)((char *)prp + size);
		while (lp < record_lim) {
			*lp = psdb_pton(*lp);
			*lp++;
		}
	}

	return(err);
}

uint32_t
rmapi_get(fd, ptr, len)
rmapi_fd_t fd;
void *ptr;
uint32_t len;
{
	PSDB_RHDR *pprp;
	uint32_t *lp, *record_lim, retval;
	int32_t record_size;

	if (len < sizeof(PSDB_RHDR))
		return(RMAPI_INCORRECT_SIZE);

	retval = psdb_io_get(fd, ptr, len);
	if (retval != 0)
		return(retval);

	pprp = (PSDB_RHDR *) ptr;

	record_size = psdb_pton(pprp->Size);

	lp = (uint32_t *) ptr;

	record_lim = (uint32_t *)((char *)lp + record_size);
	while (lp < record_lim)
		*lp++ = psdb_pton(*lp);

	return(0);
}

pdk_psdb_rhdr_t *
rmapi_to_psdb(opcode, size, param)
uint32_t opcode;
int32_t size;
void *param;
{
	PSDB_RHDR *prp;
	uint32_t *lp, *record_lim, tmp;
	
	prp = (PSDB_RHDR *) param;
	prp->Type = psdb_ntop(opcode);
	prp->Size = psdb_ntop(size);

	lp = (uint32_t *) (prp+1);
	record_lim = (uint32_t *)((char *)lp + size - sizeof(PSDB_RHDR));
	while (lp < record_lim) {
		*lp = psdb_ntop(*lp);
		lp++;
	}
	return((pdk_psdb_rhdr_t *) prp);
}
