/*==========================================================================
**
**	rmapi.h
**
**	RMAPI library functions
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	ram	01/12/99 v1.0	Created
**	cbp	10/11/99 v2.0	Changed for RMAPI
**
**==========================================================================
**
**	Notes:
**==========================================================================
*/

#ifndef PDK_RMAPI_H
#define PDK_RMAPI_H
#include <pdk_psdb.h>
#include <psdb_io.h>

/* FD functions */
rmapi_fd_t rmapi_open(char *dev, uint32_t flags);
uint32_t rmapi_close(rmapi_fd_t fd);
uint32_t rmapi_get(rmapi_fd_t fd, void *ptr, uint32_t len);
uint32_t rmapi_ctl(rmapi_fd_t fd, uint32_t opcode, void *param);

/* rmapi_ctl function and structures */
#define RMAPI_CTL_CREATE_TRUNK		PSDB_TYPE_CREATE_TRUNK
#define RMAPI_CTL_TEARDOWN_TRUNK	PSDB_TYPE_TEARDOWN_TRUNK
#define RMAPI_CTL_CREATE_FAILOVER	PSDB_TYPE_CREATE_FAILOVER
#define RMAPI_CTL_TEARDOWN_FAILOVER	PSDB_TYPE_TEARDOWN_FAILOVER
#define RMAPI_CTL_CONFIGURATION		PSDB_TYPE_CONFIGURATION
#define RMAPI_CTL_SET_MODE		PSDB_TYPE_SET_MODE
#define RMAPI_CTL_CLEAR_MODE		PSDB_TYPE_CLEAR_MODE
#define RMAPI_CTL_FAILOVER		PSDB_TYPE_FAILOVER
#define RMAPI_CTL_FAILOVER_NOTIFICATION	PSDB_TYPE_FAILOVER_NOTIFICATION
#define RMAPI_CTL_FAULT_NOTIFICATION	PSDB_TYPE_FAULT_NOTIFICATION
#define RMAPI_CTL_ERROR			PSDB_TYPE_ERROR
#define RMAPI_CTL_SUCCESS		PSDB_TYPE_SUCCESS
#define RMAPI_CTL_TEST			PSDB_TYPE_TEST

#define RMAPI_CTL_HAL_GET_STATE		PSDB_TYPE_HAL_GET_STATE
#define RMAPI_CTL_HAL_SPEEDOMETER_GET	PSDB_TYPE_HAL_SPEEDOMETER_GET
#define RMAPI_CTL_HAL_SPEEDOMETER_NOTIFICATION	PSDB_TYPE_HAL_SPEEDOMETER_NOTIFICATION
#define RMAPI_CTL_HAL_SPEEDOMETER_SET_PARAMS PSDB_TYPE_HAL_SPEEDOMETER_SET_PARAMS
#define RMAPI_CTL_HAL_FAULT_NOTIFICATION_SET PSDB_TYPE_HAL_FAULT_NOTIFICATION_SET
#define RMAPI_CTL_HAL_FAULT_NOTIFICATION_CLEAR PSDB_TYPE_HAL_FAULT_NOTIFICATION_CLEAR
#define RMAPI_CTL_HAL_FAULT_NOTIFICATION PSDB_TYPE_HAL_FAULT_NOTIFICATION

#define RMAPI_CTL_PAL_SET_MODE		PSDB_TYPE_PAL_SET_MODE
#define RMAPI_CTL_PAL_CLEAR_MODE	PSDB_TYPE_PAL_CLEAR_MODE
#define RMAPI_CTL_PAL_GET_HW_INFO	PSDB_TYPE_PAL_GET_HW_INFO


#define RMAPI_READ_FLAG			PSDB_READ_FLAG

typedef struct rmapi_create_trunk {
	PSDB_RHDR	rct_header;
	int32_t		rct_dev_id;
	int32_t		rct_num_ppa;
	int32_t		rct_ppa_array[NUM_PORTS];
} rmapi_create_trunk_t;

typedef struct rmapi_teardown_trunk {
	PSDB_RHDR	rtt_header;
	int32_t		rtt_dev_id;
} rmapi_teardown_trunk_t;

typedef struct rmapi_create_failover {
	PSDB_RHDR	rcf_header;
	int32_t		rcf_dev_id;
	int32_t		rcf_num_ppa;
	int32_t		rcf_ppa_array[NUM_PORTS];
} rmapi_create_failover_t;

typedef struct rmapi_teardown_failover {
	PSDB_RHDR	rtf_header;
	int32_t		rtf_dev_id;
} rmapi_teardown_failover_t;

typedef struct rmapi_configuration {
	PSDB_RHDR		rc_header;
	int32_t			rc_dev_id;
	int32_t			rc_ppa;
	uint32_t		rc_type;
	rlk_configuration_t	rc_configuration;
} rmapi_configuration_t;

#define RMAPI_CONFIG_TRUNK_TYPE PSDB_CONFIG_TRUNK_TYPE
#define RMAPI_CONFIG_FAILOVER_TYPE PSDB_CONFIG_FAILOVER_TYPE

typedef struct rmapi_set_mode {
	PSDB_RHDR	rsm_header;
	int32_t		rsm_dev_id;
	uint32_t	rsm_mode;
	uint32_t	rsm_time;
} rmapi_set_mode_t;

typedef struct rmapi_clear_mode {
	PSDB_RHDR	rcm_header;
	int32_t		rcm_dev_id;
	uint32_t	rcm_mode;
} rmapi_clear_mode_t;

typedef struct rmapi_failover {
	PSDB_RHDR	rf_header;
	int32_t		rf_dev_id;
	uint32_t	rf_to_ppa;
} rmapi_failover_t;

typedef struct rmapi_failover_notification {
	PSDB_RHDR	rfn_header;
	int32_t		rfn_dev_id;
	int32_t		rfn_from_ppa;
	int32_t		rfn_to_ppa;
} rmapi_failover_notification_t;

typedef struct rmapi_fault_notification {
	PSDB_RHDR	rftn_header;
	int32_t		rftn_dev_id;
	int32_t		rftn_fault_ppa;
	uint32_t	rftn_internal_fault;  /* 0 indicates external fault */
	uint32_t	rftn_on;
} rmapi_fault_notification_t;

typedef struct rmapi_test {
	PSDB_RHDR	rt_header;
	int32_t		rt_dev_id;
	uint32_t	rt_test;
} rmapi_test_t;

typedef struct rmapi_hal_get_state {
	PSDB_RHDR	rhgs_header;
	int32_t		rhgs_ppa;
	int32_t		rhgs_type;
	hal_state_t	rhgs_state;
} rmapi_hal_get_state_t;

typedef struct rmapi_hal_speed_get {
	PSDB_RHDR		rhsg_header;
	int32_t			rhsg_ppa;
	hal_speedometer_t	rhsg_speed;
} rmapi_hal_speedometer_get_t;

typedef struct rmapi_hal_speedometer_notification {
	PSDB_RHDR		rhsn_header;
	int32_t			rhsn_ppa;
	hal_speedometer_t	rhsn_speed;
} rmapi_hal_speedometer_notification_t;

typedef struct {
	PSDB_RHDR	rhf_header;
	int32_t		rhf_ppa;
	uint32_t	rhf_internal_fault;  /* 0 indicates external fault */
	uint32_t	rhf_on;
} rmapi_hal_fault_t;

#define RMAPI_GREATER_THAN_OR_EQUAL	PSDB_GREATER_THAN_OR_EQUAL
#define RMAPI_LESS_THAN			PSDB_LESS_THAN

typedef struct rmapi_hal_speedometer_set_params {
	PSDB_RHDR		rhss_header;
	int32_t			rhss_ppa;
	hal_speedometer_t	rhss_speed;
} rmapi_hal_speedometer_set_params_t;

typedef struct rmapi_pal_set_mode {
	PSDB_RHDR	rpsm_header;
	int32_t		rpsm_dev_id;
	uint32_t	rpsm_mode;
	uint32_t	rpsm_signal;
} rmapi_pal_set_mode_t;

typedef struct rmapi_pal_clear_mode {
	PSDB_RHDR	rpcm_header;
	int32_t		rpcm_dev_id;
	uint32_t	rpcm_mode;
} rmapi_pal_clear_mode_t;

typedef struct rmapi_pal_get_hw_info {
	PSDB_RHDR	rpghi_header;
	int32_t		rpghi_ppa;
	pal_hw_info_t	rpghi_info;
} rmapi_pal_get_hw_info_t;

typedef	union {
	rmapi_failover_notification_t rfn;
	rmapi_fault_notification_t rftn;
	rmapi_hal_speedometer_notification_t rh;
	rmapi_hal_fault_t rhf;
} rmapi_general_notification_t;

typedef struct rmapi_hdr {
	PSDB_RHDR	rh_header;
	int32_t		rh_dev_id;
} rmapi_hdr_t;


/* Test Codes */
#define RMAPI_TEST_SUCCESS	PSDB_TEST_SUCCESS
#define RMAPI_TEST_ERROR	PSDB_TEST_ERROR	
#define RMAPI_TEST_ASYNC	PSDB_TEST_ASYNC

/* Error Codes */
#define RMAPI_PROTOCOL_ERROR	PSDB_PROTOCOL_ERROR 
#define RMAPI_BAD_PARAM		PSDB_BAD_PARAM
#define RMAPI_INCORRECT_SIZE	PSDB_INCORRECT_SIZE
#define RMAPI_BAD_DEVICE_ID	PSDB_BAD_DEVICE_ID
#define RMAPI_TRUNK_IN_USE	PSDB_TRUNK_IN_USE
#define RMAPI_FAILOVER_IN_USE	PSDB_FAILOVER_IN_USE
#define RMAPI_PPA_OUT_RANGE	PSDB_PPA_OUT_RANGE
#define RMAPI_LAYER_PUSH_ERROR	PSDB_LAYER_PUSH_ERROR
#define RMAPI_SYSTEM_ERROR	PSDB_SYSTEM_ERROR

#endif /* PDK_RMAPI_H */

